#include "filesystemwidget.h"
#include "ui_filesystemwidget.h"

FileSystemWidget::FileSystemWidget(QWidget *parent, bool standalone, QString openLocation) :
    QWidget(parent),
    ui(new Ui::FileSystemWidget)
{
	setWindowIcon(QIcon(":/icons/32x32/folder.png"));
    model = 0;
	searchProgress = 0;
	searching = false;
    cut = false;
    ui->setupUi(this);

    remoteClientManager = new RemoteClientManager();
    if ( ! standalone )
    {
        QTimer::singleShot( NETWORK_START_WAIT_TIME, remoteClientManager, SLOT(start()));
        connect( remoteClientManager, SIGNAL(update()), this, SLOT(updatePlaces()));
		connect( remoteClientManager, SIGNAL(update()), this, SLOT(setHome()));
    }

    fillPlaces();

    initUi(openLocation);

    setEnabledButtons();
}

FileSystemWidget::~FileSystemWidget()
{
	delete ui;
}

void FileSystemWidget::initUi(QString openLocation)
{
    menu = new QMenu(this);
    noSelectionMenu = new QMenu(this);
    setMenu();
    setNoSelectionMenu();

    ui->placesView->setModel( placesModel );
    ui->placesView->setIconSize( QSize( ICON_SIZE, ICON_SIZE ) );

    initModel();

    ui->listView->setItemsExpandable( false );
    ui->listView->setIconSize( QSize( ICON_SIZE * 0.35, ICON_SIZE * 0.35 ));
    ui->listView->setStyleSheet( "QTreeView::branch { border-image: url(nonexistent.png); }");
    ui->listView->setSortingEnabled( true );

    ui->iconView->setViewMode(QListView::IconMode);
    ui->iconView->setIconSize( QSize(ICON_SIZE, ICON_SIZE) );
    ui->iconView->setGridSize( QSize(ICON_SIZE * 2, ICON_SIZE * 2) );
    ui->iconView->setResizeMode( QListView::Adjust );
    ui->iconView->setMovement(QListView::Snap);

    if ( openLocation.isEmpty() )
    {
        setHome();
    } else {
        setIndex( model->index( openLocation ) );
    }

    ui->listView->hide();

    QFont font;
    font.setBold( true );
    ui->addressLineEdit->setFont( font );
    ui->addressLineEdit->addAction(QIcon(ICON_DIR_32 + QString(QDir::separator()) + "folder-docs.png"), QLineEdit::LeadingPosition);
    ui->addressLineEdit->setReadOnly( true );

    connect( ui->iconViewButton, SIGNAL(pressed()), ui->iconView, SLOT(show()));
    connect( ui->iconViewButton, SIGNAL(pressed()), ui->listView, SLOT(hide()));
    connect( ui->iconViewButton, SIGNAL(pressed()), this, SLOT(setEnabledButtons()));
    connect( ui->iconViewButton, SIGNAL(pressed()), this, SLOT(changeView()));

    connect( ui->listViewButton, SIGNAL(pressed()), ui->listView, SLOT(show()));
    connect( ui->listViewButton, SIGNAL(pressed()), ui->iconView, SLOT(hide()));
    connect( ui->listViewButton, SIGNAL(pressed()), this, SLOT(setEnabledButtons()));
    connect( ui->listViewButton, SIGNAL(pressed()), this, SLOT(changeView()));

    connect( ui->upButton, SIGNAL(pressed()), this, SLOT(upButtonPressed()));

    connect( ui->listView, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(doubleClicked(QModelIndex)));
    connect( ui->listView, SIGNAL(pressed(QModelIndex)), this, SLOT(mousePressed(QModelIndex)));
    ui->listView->setContextMenuPolicy(Qt::CustomContextMenu);
    connect( ui->listView, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(menuRequested(QPoint)));

    connect( ui->iconView, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(doubleClicked(QModelIndex)));
    connect( ui->iconView, SIGNAL(pressed(QModelIndex)), this, SLOT(mousePressed(QModelIndex)));
    ui->iconView->setContextMenuPolicy(Qt::CustomContextMenu);
    connect( ui->iconView, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(menuRequested(QPoint)));

    connect( ui->placesView, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(placesDoubleClicked(QModelIndex)));
    connect( ui->placesView, SIGNAL(clicked(QModelIndex)), this, SLOT(placesDoubleClicked(QModelIndex)));

    connect( ui->renameButton, SIGNAL(pressed()), this, SLOT(renameSlot()));
    connect( ui->newFolderButton, SIGNAL(pressed()), this, SLOT(newFolderSlot()));
    connect( ui->cutButton, SIGNAL(pressed()), this, SLOT(cutSlot()));
    connect( ui->copyButton, SIGNAL(pressed()), this, SLOT(copySlot()));
    connect( ui->pasteButton, SIGNAL(pressed()), this, SLOT(pasteSlot()));
    connect( ui->removeButton, SIGNAL(pressed()), this, SLOT(removeSlot()));
}

void FileSystemWidget::setHome()
{
	setIndex( model->index(Settings::homePath()) );
}

void FileSystemWidget::directoryLoaded(QString dir)
{
    QString path = dir;
    updatePath( & path );
    connectModelSignals();
    ui->fileInfo->setText(tr("Finished loading folder %1").arg(path));
}

void FileSystemWidget::init()
{
    ui->splitter->setStretchFactor(0, 1);
    ui->splitter->setStretchFactor(1, 4);
    ui->listView->setColumnWidth(0, qApp->desktop()->width() / 3);
}

void FileSystemWidget::fillPlaces()
{
    placesModel = new QStandardItemModel;
    updatePlaces();
}

QString FileSystemWidget::remotePlaceName(int i)
{
	QString place = tr("REMOTE > ") + remoteClientManager->getNames().at(i);
	QString drive = remoteClientManager->getDrives().at(i);
	if ( ! drive.isEmpty() )
	{
		place = place + " (" + drive.toUpper() + ")";
	}
	return place;
}

void FileSystemWidget::updatePlaces()
{
    placesModel->clear();
    insertPlace( QIcon(ICON_DIR_64 + QString(QDir::separator()) + "folder-home.png"), qgetenv( "USERNAME" ) );
    for (int i = 0; i < remoteClientManager->getNames().count(); i++)
    {
		if ( Settings::homePath().toLower() != remoteClientManager->getDrives().at(i).toLower() )
		{
			insertPlace( QIcon(ICON_DIR_64 + QString(QDir::separator()) + "folder.png"), remotePlaceName(i));
		}
    }
	if (QFile::exists( Settings::homePath() + QDir::separator() + "Dropbox"))
    {
        insertPlace( QIcon(ICON_DIR_APPS + QString(QDir::separator()) + "dropbox.png"), tr("Dropbox") );
    }
	if (QFile::exists( Settings::homePath() + QDir::separator() + "Google Drive"))
    {
        insertPlace( QIcon(ICON_DIR_APPS + QString(QDir::separator()) + "google-drive.png"), tr("Google Drive") );
    }
}

void FileSystemWidget::insertPlace( QIcon icon, QString name )
{
    QFont font;
    font.setBold( true );
    QStandardItem * item = new QStandardItem( icon, name );
    item->setEditable( false );
    item->setFont( font );
    placesModel->insertRow( placesModel->rowCount(), item );
}

void FileSystemWidget::disconnectModelSignals()
{
    disconnect(model, SIGNAL(rootPathChanged(QString)), ui->listView->selectionModel(), SLOT(clearSelection()));
    disconnect(model, SIGNAL(rootPathChanged(QString)), ui->iconView->selectionModel(), SLOT(clearSelection()));
    disconnect( ui->iconView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)), this, SLOT(setEnabledButtons()));
    disconnect( ui->listView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)), this, SLOT(setEnabledButtons()));
}

void FileSystemWidget::connectModelSignals()
{
    connect(model, SIGNAL(rootPathChanged(QString)), ui->listView->selectionModel(), SLOT(clearSelection()));
    connect(model, SIGNAL(rootPathChanged(QString)), ui->iconView->selectionModel(), SLOT(clearSelection()));
    connect( ui->iconView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)), this, SLOT(setEnabledButtons()));
    connect( ui->listView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)), this, SLOT(setEnabledButtons()));
}

void FileSystemWidget::initModel()
{
    if ( model != 0)
    {
        disconnect(model, SIGNAL(directoryLoaded(QString)), this, SLOT(directoryLoaded(QString)));
        disconnectModelSignals();
    } else {
        iconProvider = new FileIconProvider();
    }

    QFileSystemModel * newModel = new QFileSystemModel;
    newModel->setResolveSymlinks( false );
    newModel->setIconProvider( iconProvider );
	ui->listView->reset();
    ui->listView->setModel( newModel );
	ui->iconView->reset();
    ui->iconView->setModel( newModel );
    if ( model != 0)
    {
        delete model;
    }
    model = newModel;
    connect(model, SIGNAL(directoryLoaded(QString)), this, SLOT(directoryLoaded(QString)));
    connectModelSignals();
    setEnabledButtons();
}

void FileSystemWidget::placesDoubleClicked(QModelIndex index)
{
    initModel();

	if (index.isValid())
	{
		setPlaceIndex( index );
	}
}

void FileSystemWidget::setPlaceIndex( QModelIndex index )
{
    QStandardItem * item = placesModel->itemFromIndex( index );
    if ( item != 0)
    {
        QString name = item->text();
        if ( name == tr("Dropbox") )
        {
			setIndex( model->index(Settings::homePath() + QDir::separator() + "Dropbox") );
        } else if ( name == tr("Google Drive") ) {
			setIndex( model->index(Settings::homePath() + QDir::separator() + "Google Drive") );
        } else if ( name == qgetenv( "USERNAME" ) ) {
            setHome();
        } else {
            for (int i = 0; i < remoteClientManager->getNames().count(); i++)
            {
				if ( name == remotePlaceName(i) )
                {
                    setIndex( model->index( remoteClientManager->getDrives().at(i) ) );
                    return;
                }
            }
        }
    }
}

void FileSystemWidget::upButtonPressed()
{
    QModelIndex index = model->index( model->rootPath() );
    bool allow = true;
	qDebug() << Settings::homePath();
	qDebug() <<  model->rootPath();
	if ( QFileInfo(model->rootPath()).absoluteFilePath() == QFileInfo(Settings::homePath()).absoluteFilePath() )
    {
        allow = false;
    }
    for (int i = 0; i < remoteClientManager->getDrives().count(); i++)
    {
        QString path = model->rootPath();
        if ( path.right(1) == "/" )
        {
            path.chop(1);
        }
        QString drive = remoteClientManager->getDrives().at(i);
        if ( model->rootPath() == drive || path == drive )
        {
            allow = false;
        }
    }
    if (allow)
    {
        QString parentPath = model->filePath( model->parent( index ) );
        initModel();
        setIndex( model->index( parentPath ) );
    }
}

void FileSystemWidget::setIndex( QModelIndex index )
{
	if ( ! model->fileInfo( index ).exists() )
	{
		remoteClientManager->stop();
		remoteClientManager->start();
		updatePlaces();
		setHome();
		return;
	}
    ui->listView->selectionModel()->clearSelection();
    ui->iconView->selectionModel()->clearSelection();
    disconnectModelSignals();
    QString path =  model->filePath( index );
    QString updatedPath = path;
    updatePath( &updatedPath );
    ui->fileInfo->setText( tr("Loading folder %1...").arg(updatedPath));
    model->setRootPath( path );
    ui->listView->setRootIndex( index );
    ui->iconView->setRootIndex( index );
    ui->addressLineEdit->setText( updatedPath );
}

void FileSystemWidget::updatePath(QString * path)
{
	path->replace( QRegExp("^" + Settings::homePath()), qgetenv( "USERNAME") );
    for (int i = 0; i < remoteClientManager->getDrives().count(); i++)
    {
		path->replace( QRegExp("^" + remoteClientManager->getDrives().at(i)), tr("REMOTE > ") + remoteClientManager->getNames().at(i) );
    }
    if ( path->right(1) == "/" )
    {
        path->chop(1);
    }
}

void FileSystemWidget::doubleClicked(QModelIndex index)
{
	if ( index.isValid() )
	{
		if ( ! model->fileInfo( index ).isSymLink() )
		{
			if ( model->fileInfo( index ).isDir() )
			{
				setIndex( index );
			}
			if ( model->fileInfo( index ).isFile() )
			{
				emit openFile( model->filePath( index ) );
			}
		} else {
			setHome();
		}
	}
}

void FileSystemWidget::mousePressed(QModelIndex index)
{
    // Right click
    if (QApplication::mouseButtons() == Qt::RightButton)
    {
        showMenu( index );
    }
}

void FileSystemWidget::menuRequested(const QPoint pos)
{
    if (ui->listView->isVisible())
    {
        if (ui->listView->indexAt(pos) == QModelIndex())
        {
            showNoSelectionMenu();
        }
    } else {
        if (ui->iconView->indexAt(pos) == QModelIndex())
        {
            showNoSelectionMenu();
        }
    }
}

void FileSystemWidget::showMenu( QModelIndex index )
{
	openWithAction->setVisible( false );
	if (index.isValid())
	{
		if ( model->fileInfo( index ).isFile() )
		{
			openWithAction->setVisible( true );
		}
	}
    menu->exec(QCursor::pos());
}

void FileSystemWidget::showNoSelectionMenu()
{
    noSelectionMenu->exec(QCursor::pos());
}

void FileSystemWidget::close()
{
	if ( model != 0 )
	{
		delete model;
	}
    remoteClientManager->stop();
    delete remoteClientManager;
    QWidget::close();
}

void FileSystemWidget::setMenu()
{
    renameAction = new QAction( QIcon(ICON_DIR_32 + QString(QDir::separator()) + "rename.png"), tr("&Rename"), this );
    menu->addAction( renameAction );
    connect( renameAction, SIGNAL(triggered()), this, SLOT(renameSlot()));

    cutAction = new QAction( QIcon(ICON_DIR_32 + QString(QDir::separator()) + "cut.png"), tr("Cu&t"), this );
    menu->addAction( cutAction );
    connect( cutAction, SIGNAL(triggered()), this, SLOT(cutSlot()));

    copyAction = new QAction( QIcon(ICON_DIR_32 + QString(QDir::separator()) + "copy.png"), tr("&Copy"), this );
    menu->addAction( copyAction );
    connect( copyAction, SIGNAL(triggered()), this, SLOT(copySlot()));

    removeAction = new QAction( QIcon(ICON_DIR_32 + QString(QDir::separator()) + "remove.png"), tr("&Delete"), this );
    menu->addAction( removeAction );
    connect( removeAction, SIGNAL(triggered()), this, SLOT(removeSlot()));

	openWithAction = new QAction( QIcon(ICON_DIR_32 + QString(QDir::separator()) + "openwith.png"), tr("&Open with..."), this );
	menu->addAction( openWithAction );
	connect( openWithAction, SIGNAL(triggered()), this, SLOT(openWithSlot()));
}

void FileSystemWidget::setNoSelectionMenu()
{
    newFolderAction = new QAction( QIcon(ICON_DIR_32 + QString(QDir::separator()) + "folder-new.png"), tr("&New folder"), this );
    noSelectionMenu->addAction( newFolderAction );
    connect( newFolderAction, SIGNAL(triggered()), this, SLOT(newFolderSlot()));

    pasteAction = new QAction( QIcon(ICON_DIR_32 + QString(QDir::separator()) + "paste.png"), tr("&Paste"), this );
    noSelectionMenu->addAction( pasteAction );
    connect( pasteAction, SIGNAL( triggered()), this, SLOT(pasteSlot()));

	findFileAction = new QAction( QIcon(ICON_DIR_32 + QString(QDir::separator()) + "search.png"), tr("&Find file"), this );
	noSelectionMenu->addAction( findFileAction );
	connect( findFileAction, SIGNAL( triggered()), this, SLOT(findFileSlot()));
}

void FileSystemWidget::cutSlot()
{
    copiedFile = model->fileInfo(selectedIndex);
    cut = true;
    setEnabledButtons();
}

void FileSystemWidget::copySlot()
{
    copiedFile = model->fileInfo(selectedIndex);
    cut = false;
    setEnabledButtons();
}

void FileSystemWidget::pasteSlot()
{
    if ( QFile::exists( model->rootPath() + QDir::separator() + copiedFile.fileName() ) )
    {
        QMessageBox::critical( this, tr("Cannot paste"), tr("The target you have selected currently exists."), QMessageBox::Ok, QMessageBox::NoButton);
    } else {
        copyFile();
        if ( cut )
        {
            selectedIndex = model->index(copiedFile.absoluteFilePath());
            removeFile();
            cut = false;
        }
        copiedFile = QFileInfo();
        selectedIndex = QModelIndex();
        setEnabledButtons();
    }
}

void FileSystemWidget::removeSlot()
{
    bool doIt = true;

    QString title;
    QString message;
    if ( model->fileInfo( selectedIndex ).isDir() )
    {
        title = tr("Remove folder");
        message = tr("This folder is to be removed with all its contents. This operation cannot be undone. Are you sure you want to remove this folder?");
    } else {
        title = tr("Remove file");
        message = tr("This file is to be removed. This operation cannot be undone. Are you sure you want to remove this file?");
    }
    doIt = QMessageBox::warning( this, title, message, QMessageBox::Yes, QMessageBox::No ) == QMessageBox::Yes;

    if (doIt)
    {
        removeFile();
        selectedIndex = QModelIndex();
        ui->listView->selectionModel()->clearSelection();
        ui->iconView->selectionModel()->clearSelection();
    }
}

void FileSystemWidget::openFile()
{
	QAction * _sender = (QAction*)sender();
	QStringList data = _sender->data().toStringList();
	if (data.count() == 2)
	{
		emit openFile(data[0], data[1]);
	}
}

void FileSystemWidget::openWithSlot()
{
	QStringList applications;
	QFileInfo file = model->fileInfo( selectedIndex );
	if ( file.isFile() )
	{
		QMenu * openWithMenu = new QMenu(this);
		QString extension = "." + QFileInfo( file ).suffix();
		if (Application::mimeType(file) == "recursivelink")
		{
			QMessageBox::critical ( 0, tr("Link error"), tr("This file is a recursive set of links"));
			return;
		}
		applications += Application::getValidApplications();
		QString myApp = Application::preferredProgram(extension);
		if (! myApp.isEmpty())
		{
			applications.move( applications.indexOf(myApp), 0);
		}

		for ( int i = 0; i < applications.count(); i++ )
		{
			qDebug("Including application on menu...");
			if ( !  myApp.isEmpty() && i == 1)
			{
				openWithMenu->addSeparator();
			}
			QAction * action = openWithMenu->addAction( Application::getIcon(applications[i]), Application::getText(applications[i]), this, SLOT(openFile()));
			QStringList data;
			data += file.absoluteFilePath();
			data += applications[i];
			action->setData( data );
		}
		openWithMenu->exec(QCursor::pos());
		connect(openWithMenu, SIGNAL(aboutToHide()), openWithMenu, SLOT(deleteLater()));
	}
}

void FileSystemWidget::newFolderSlot()
{
    QString newFolderName = QInputDialog::getText( this, tr("New folder"), tr("Please, enter new folder name"));
    if ( ! newFolderName.isEmpty() )
    {
        if ( QFile::exists( model->rootPath() + QDir::separator() + newFolderName))
        {
            QMessageBox::critical( this, tr("Cannot create folder"), tr("The folder name you have selected currently exists. Please, select a different folder name"), QMessageBox::Ok, QMessageBox::NoButton);
        } else {
            model->mkdir( model->index( model->rootPath()), newFolderName );
        }
    }
}

void FileSystemWidget::renameSlot()
{
    if ( model->fileInfo( selectedIndex ).isDir() )
    {
        QString newName = QInputDialog::getText( this, tr("Rename"), tr("Please, enter new folder name"), QLineEdit::Normal, model->fileName( selectedIndex ) );
        if ( newName !=  model->fileName( selectedIndex ) && ! newName.isEmpty() )
        {
            if ( QFile::exists( model->rootPath() + QDir::separator() + newName))
            {
                QMessageBox::critical( this, tr("Cannot rename folder"), tr("The folder name you have selected currently exists. Please, select a different folder name"), QMessageBox::Ok, QMessageBox::NoButton);
            } else {
                renameFile( newName );
            }
        }
    } else {
        QString newName = QInputDialog::getText( this, tr("Rename"), tr("Please, enter new file name"), QLineEdit::Normal, model->fileName( selectedIndex ) );
        if ( newName !=  model->fileName( selectedIndex ) && ! newName.isEmpty() )
        {
            if ( QFile::exists( model->rootPath() + QDir::separator() + newName))
            {
                QMessageBox::critical( this, tr("Cannot rename file"), tr("The file name you have selected currently exists. Please, select a different file name"), QMessageBox::Ok, QMessageBox::NoButton);
            } else {
                renameFile( newName );
            }
        }
    }
}

void FileSystemWidget::findFileSlot()
{
	bool alreadyFound = false;
	QString ret;
	QString searchString = QInputDialog::getText( this, tr("Find file"), tr("Please, enter the search string"), QLineEdit::Normal );
	if ( ! searchString.isEmpty() )
	{
		bool cancelSearch = false;
		mutex.lock();
		searching = true;
		mutex.unlock();
		if ( searchProgress != 0 )
		{
			delete searchProgress;
		}
		searchProgress = new QProgressDialog(QString::null, tr("Cancel search"), 0, 0, this);
		QLabel * searchLabel = new QLabel(tr("Search progress..."));
		searchLabel->setFixedWidth(300);
		searchLabel->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
		searchProgress->setLabel( searchLabel );
		searchProgress->setWindowTitle(tr("Find file"));
		searchProgress->setWindowIcon(QIcon(":/icons/32x32/search.png"));
		connect(searchProgress, SIGNAL(canceled()), this, SLOT(cancelSearch()));
		searchProgress->show();
		qApp->processEvents();

		QDirIterator it(model->rootPath(), QDirIterator::Subdirectories);
		while (it.hasNext() && ! cancelSearch)
		{
			it.next();
			searchProgress->setLabelText(tr("File name: %1").arg(it.fileName()));
			if (it.fileName().contains( searchString, Qt::CaseInsensitive) )
			{
				ret = it.filePath();
				if ( ! ret.isEmpty() )
				{
					QModelIndex index = model->index(ret);
					if ( index.isValid() )
					{
						alreadyFound = true;
						if (model->fileInfo(index).isDir())
						{
							setIndex(index);
						} else {
							QModelIndex parent = index.parent();
							setIndex(parent);
							if (ui->iconView->isVisible())
							{
								ui->iconView->scrollTo( index );
							}
							if (ui->listView->isVisible())
							{
								ui->listView->scrollTo( index );
							}
							searchProgress->hide();
							if ( QMessageBox::information( this,
												   tr("Find file"),
												   tr("Found file '%1'. Do you want to continue searching?").arg( ret ),
												   QMessageBox::Yes,
												   QMessageBox::No) == QMessageBox::No )
							{
								searchProgress->cancel();
								delete searchProgress;
								searchProgress = 0;
								return;
							} else {
								searchProgress->show();
							}
						}
					}
				}
			}
			mutex.lock();
			cancelSearch = ! searching;
			mutex.unlock();
			qApp->processEvents();
		}
	} else {
		return;
	}

	if (searchProgress != 0)
	{
		searchProgress->cancel();
		delete searchProgress;
		searchProgress = 0;
	}

	if ( ! alreadyFound )
	{
		QMessageBox::warning( this, tr("Find file"), tr("No file or folder found matching your search"));
	} else {
		QMessageBox::information( this, tr("Find file"), tr("No more files or folders found"));
	}
}

void FileSystemWidget::cancelSearch()
{
	mutex.lock();
	searching = false;
	mutex.unlock();
}

QString FileSystemWidget::findFile(QString searchString)
{
	QDirIterator it(model->rootPath(), QDirIterator::Subdirectories);
	while (it.hasNext())
	{
		it.next();
		if (it.fileName().contains( searchString, Qt::CaseInsensitive) )
		{
			return it.filePath();
		}
	}
	return QString();
}

void FileSystemWidget::setEnabledButtons()
{
    if (copiedFile.absoluteFilePath().isEmpty())
    {
        pasteAction->setEnabled( false );
        ui->pasteButton->setEnabled( false );
    } else {
        pasteAction->setEnabled( true );
        ui->pasteButton->setEnabled( true );
    }
    if (ui->listView->isVisible())
    {
        ui->listViewButton->setEnabled( false );
        ui->iconViewButton->setEnabled( true );
    } else {
        ui->listViewButton->setEnabled( true );
        ui->iconViewButton->setEnabled( false );
    }
    if ( ui->listView->selectionModel()->hasSelection() || ui->iconView->selectionModel()->hasSelection() )
    {
        if ( ui->listView->selectionModel()->hasSelection() )
        {
            selectedIndex = ui->listView->selectionModel()->selectedIndexes()[0];
        } else {
            selectedIndex = ui->iconView->selectionModel()->selectedIndexes()[0];
        }
        renameAction->setEnabled( true );
        ui->renameButton->setEnabled( true );
        copyAction->setEnabled( true );
        ui->copyButton->setEnabled( true );
        cutAction->setEnabled( true );
        ui->cutButton->setEnabled( true );
        removeAction->setEnabled( true );
        ui->removeButton->setEnabled( true );
    } else {
        selectedIndex = QModelIndex();
        renameAction->setEnabled( false );
        ui->renameButton->setEnabled( false );
        copyAction->setEnabled( false );
        ui->copyButton->setEnabled( false );
        cutAction->setEnabled( false );
        ui->cutButton->setEnabled( false );
        removeAction->setEnabled( false );
        ui->removeButton->setEnabled( false );
    }
}

void FileSystemWidget::changeView()
{
    if (  ui->listView->selectionModel()->hasSelection() )
    {
        ui->iconView->selectionModel()->select( ui->listView->selectionModel()->selection(), QItemSelectionModel::ClearAndSelect );
    } else {
        ui->listView->selectionModel()->select( ui->iconView->selectionModel()->selection(), QItemSelectionModel::ClearAndSelect );
    }
}


void FileSystemWidget::copyFile()
{
    if ( copiedFile.isDir() )
    {
        copyRecursively( copiedFile.absoluteFilePath(), model->rootPath() + QDir::separator() + copiedFile.fileName() );
    } else {
        QFile file( copiedFile.absoluteFilePath() );
        if ( ! file.copy( model->rootPath() + QDir::separator() +   copiedFile.fileName() ) )
        {
            cut = false;
            QMessageBox::critical( this, tr("Cannot paste file"), tr("Something went wrong when pasting file. Please, contact system administrator"), QMessageBox::Ok, QMessageBox::NoButton);
        }
    }
}

void FileSystemWidget::removeFile()
{
    if ( model->fileInfo( selectedIndex ).isDir() )
    {
        QDir( model->fileInfo( selectedIndex ).absoluteFilePath() ).removeRecursively();
    }
    model->remove( selectedIndex );
}

void FileSystemWidget::renameFile(QString newName)
{
        QDir dir(model->rootPath());
        dir.rename( model->fileName( selectedIndex ), newName );
}

bool FileSystemWidget::copyRecursively(QString sourceFile, QString targetFile)
{
    QFileInfo sourceFileInfo(sourceFile);
    if (sourceFileInfo.isDir())
    {
        QDir targetDir(targetFile);
        targetDir.cdUp();
        if ( ! targetDir.mkdir(QFileInfo(targetFile).fileName()))
        {
            return false;
        }
        QDir sourceDir(sourceFile);
        QStringList fileNames = sourceDir.entryList(QDir::Files | QDir::Dirs | QDir::NoDotAndDotDot | QDir::Hidden | QDir::System);
        foreach (const QString &fileName, fileNames)
        {
            const QString newSrcFilePath = sourceFile + QDir::separator() + fileName;
            const QString newTgtFilePath = targetFile + QDir::separator() + fileName;
            if (!copyRecursively(newSrcFilePath, newTgtFilePath))
            {
                return false;
            }
        }
    } else {
        QFile sourceQFile(sourceFile);
        QFile targetQFile(targetFile);
        QFile::Permissions permissions = sourceQFile.permissions();
        if ( ! QFile::copy(sourceFile, targetFile) )
        {
            return false;
        } else {
            targetQFile.setPermissions( permissions );
        }
    }
    return true;
}
