#include <QDir>

#include "mimeconfig.h"

MimeConfig::MimeConfig(Menu * menu) : QDialog()
{
	dontclose = true;
	setMenuWidget( menu );
	init (QString::null, menu);
}

MimeConfig::MimeConfig(QString file, QWidget * parent) : QDialog(parent)
{
	dontclose = false;
	init (file, parent);
}

void MimeConfig::init(QString file, QWidget *)
{
	layout = new QGridLayout(this);
	layout->setSpacing(10);
	layout->setMargin(20);

	backButton = new ToolButton(QIcon(ICON_DIR_32 + QString(QDir::separator()) + "back.png"), tr("Go back"), this);

	appGroup = new QComboBox(this);
	appGroup->setFixedHeight(ICON_SIZE);
	appGroup->setIconSize(QSize(ICON_SIZE / 2, ICON_SIZE / 2));
	QStringList groupIds = Application::applicationGroupIdsList();
	QStringList groups = Application::applicationGroupsList();
    for (int i = 0; i < groups.count(); i++)
	{
        QString icon = Settings::getValue( GROUPS_FILE, groupIds[i] + "/icon" );
		QString name = groups[i];
        appGroup->addItem(QIcon(ICON_DIR_GROUPS + QString(QDir::separator()) + icon + ".png"), name);
		appGroup->setItemData(appGroup->count() - 1, groupIds[i]);
 	}
	appGroup->addItem( tr("<No group>") );
	appGroup->setItemData(appGroup->count() - 1, "-1");

	applications = new QListWidget(this);
	applications->setIconSize(QSize(ICON_SIZE/2, ICON_SIZE/2));

	applicationsToSelected = new ToolButton(QIcon(ICON_DIR_32 + QString(QDir::separator()) + "forward.png"), "", this);
	applicationsToSelected->setEnabled( false );

	extensions = new QComboBox(this);
	extensions->setFixedHeight(ICON_SIZE);
	extensions->addItems(Application::getExtensionsList());

	selected = new QListWidget(this);
	selected->setIconSize(QSize(ICON_SIZE/2, ICON_SIZE/2));

	up = new ToolButton(QIcon(ICON_DIR_32 + QString(QDir::separator()) + "up.png"), "", this);
	up->setEnabled(false);
	down = new ToolButton(QIcon(ICON_DIR_32 + QString(QDir::separator()) + "down.png"), "", this);
	down->setEnabled(false);

	deleteButton = new ToolButton(QIcon(ICON_DIR_32 + QString(QDir::separator()) + "remove.png"), "", this);
	deleteButton->setEnabled(false);

	acceptButton = new ToolButton(QIcon(ICON_DIR_32 + QString(QDir::separator()) + "accept.png"), "", this);
	cancelButton = new ToolButton(QIcon(ICON_DIR_32 + QString(QDir::separator()) + "cancel.png"), "", this);

	layout->addWidget(backButton, 0, 0);
	layout->addWidget(appGroup, 1, 0);
	layout->addWidget(applications, 2, 0, 7, 1);
	layout->addWidget(applicationsToSelected, 3, 1);
	layout->addWidget(extensions, 1, 2, 1, 2);
	layout->addWidget(selected, 2, 2, 7, 2);
	layout->addWidget(up, 3, 4);
	layout->addWidget(down, 4, 4);
	layout->addWidget(deleteButton, 6, 4);
	layout->addWidget(acceptButton, 9, 3);
	layout->addWidget(cancelButton, 9, 4);

	for (int i=0; i<=4; i++)
		layout->setColumnStretch(i, 3);
	layout->setColumnStretch(1, 1);
	layout->setColumnStretch(4, 1);

    connect(backButton, SIGNAL(clicked()), this, SLOT(reject()));
	connect(appGroup, SIGNAL(activated(int)), this, SLOT(appGroupActivated(int)));
	connect(applicationsToSelected, SIGNAL(clicked()), this, SLOT(applicationsToSelectedClicked()));
	connect(extensions, SIGNAL(activated(int)), this, SLOT(extensionsActivated(int)));
	connect(up, SIGNAL(clicked()), this, SLOT(upClicked()));
	connect(down, SIGNAL(clicked()), this, SLOT(downClicked()));
	connect(deleteButton, SIGNAL(clicked()), this, SLOT(deleteSelected()));
	connect(acceptButton, SIGNAL(clicked()), this, SLOT(accept()));
	connect(cancelButton, SIGNAL(clicked()), this, SLOT(reject()));

	if (appGroup->count() > 0)
		appGroupActivated(0);

	if (! file.isEmpty())
	{
		QFileInfo fileInfo( file );
		QString extension = "." + fileInfo.suffix();
		int index = extensions->findText(extension);
		if (index == -1)
		{
			Application::addExtension(extension);
			extensions->clear();
			extensions->addItems( Application::getExtensionsList() );
			index = extensions->findText(extension);
		}
		if (index == -1)
			reject();
		updateSelected( index );
		extensions->setEnabled(false);
	} else {
		if (extensions->count() > 0)
			updateSelected(0);
	}

	connect(applications, SIGNAL(currentRowChanged (int) ), this, SLOT(applicationsChanged (int) ));
	connect(selected, SIGNAL(currentRowChanged (int) ), this, SLOT(selectedChanged (int) ));
}

void MimeConfig::accept()
{
	saveConfig(modifiedItems());
	if ( dontclose )
	{
        hide();
	} else {
		QDialog::accept();
	}
}

QStringList MimeConfig::modifiedItems()
{
	bool touched = false;
	QStringList itemList;

	int index = extensions->currentIndex();
	if (index == -1)
		return QStringList();

	QString extension = extensions->itemText(index);
	QList<Application*> apps = Application::applicationList( extension );

	int count = selected->count();

	touched = ( count != apps.count() );

	for (int i = 0; i < count; i++)
	{
		itemList.append(selected->item(i)->data(Qt::UserRole).toString());
		if (! touched)
		{
			if (itemList[i] != apps[i]->getName())
			{
				touched = true;
			}
		}
	}

	Application::deleteList( apps );

	if (touched)
	{
		return itemList;
	} else {
		return QStringList();
	}
}

void MimeConfig::saveConfig(QStringList modifiedItems)
{
	deletedItems.removeDuplicates();
	int count = modifiedItems.count();
	int dcount = deletedItems.count();

	qDebug("modifiedItems: %s", modifiedItems.join(":").toUtf8().data());
	qDebug("deletedItems: %s", deletedItems.join(":").toUtf8().data());

	if (count != 0 || dcount != 0)
	{
		QMessageBox::StandardButton res = QMessageBox::question ( this, tr("Modified configuration"), tr("Are you sure you want to change applications configuration?"), QMessageBox::Save | QMessageBox::Discard);
		if (res == QMessageBox::Save)
		{
			QStringList itemsToSave = modifiedItems;
			for (int i = 0; i < dcount; i++)
			{
				itemsToSave += "!" + deletedItems[i];
			}

			QString file = Settings::homePath() + "/.desktux/mime/extensions";
			QString key = extensions->currentText();
			qDebug("key: %s", key.toUtf8().data());
			if (! key.isEmpty() )
			{
				Settings::setValue(file, key, itemsToSave.join(":"));
			}
			emit mimeChanged();
		} else {
			qDebug("Didn't save!");
		}
	}
}

void MimeConfig::reject()
{
	if ( dontclose )
	{
        hide();
	} else {
		QDialog::reject();
	}
}

void MimeConfig::deleteSelected()
{
	QList<QListWidgetItem *> toDelete = selected->selectedItems();

	int count = toDelete.count();

	for (int i = 0; i < count; i++)
	{
		QString itemToDelete = toDelete[i]->data(Qt::UserRole).toString();

		if (deletedItems.indexOf(itemToDelete) == -1)
			deletedItems.append(itemToDelete);

		if (selectedContains(itemToDelete))
		{
			deleteApplication(itemToDelete);
		}
	}
}

void MimeConfig::appGroupActivated(int index)
{
	applications->clear();
	QString groupId = appGroup->itemData(index).toString();
	QList<Application*> apps = Application::applicationListByGroup(groupId);
	int count = apps.count();
	for (int i=0; i < count; i++)
	{
		appendApplication(apps[i], applications);
	}

    Application::deleteList( apps );
}

void MimeConfig::extensionsActivated(int index)
{
	saveConfig(modifiedItems());

	deletedItems.clear();
	appendedItems.clear();

	if (index != -1)
	{
		QString extension = extensions->itemText(index);
	}

	updateSelected(index);
}

void MimeConfig::updateSelected(int index)
{
	selected->clear();

	extensions->setCurrentIndex(index);
	QString extension = extensions->itemText(index);
	QList<Application*> apps = Application::applicationList( extension );

	QList<Application*> delApps = Application::applicationDeletedList( extension );
	int count = delApps.count();
	for (int i = 0; i < count; i++)
	{
		deletedItems.append(delApps[i]->getName());
	}

	count = apps.count();
	for (int i=0; i < count; i++)
	{
		QString itemToAdd = apps[i]->getName();
		if (deletedItems.indexOf(itemToAdd) == -1)
		{
			appendApplication(apps[i], selected);
		}
	}

	count = appendedItems.count();
	for (int i = 0; i < count; i++)
	{
		if (deletedItems.indexOf(appendedItems[i]) == -1 && ! selectedContains(appendedItems[i]))
		{
			Application * app = new Application(appendedItems[i]);
			appendApplication(app, selected);
			delete app;
		}
	}

	Application::deleteList( apps );
	Application::deleteList( delApps );
}

void MimeConfig::appendApplication(Application * application, QListWidget * listWidget)
{
	QIcon icon = application->getIcon();
	QString name = application->getName();
	QString text = Application::getText(name);
    qDebug("Appending application %s...", text.toUtf8().data());

    QListWidgetItem * item = new QListWidgetItem(icon, text, listWidget);
	item->setData( Qt::UserRole, name );
}

void MimeConfig::deleteApplication(QString application)
{
	int count = selected->count();

	for (int i = 0; i < count; i++)
	{
		if (selected->item(i)->data(Qt::UserRole).toString() == application)
		{
			QListWidgetItem * item = selected->takeItem(i);
			delete item;
			count = selected->count();
			i--;
		}
	}
	selected->setCurrentRow( -1 );
}

void MimeConfig::applicationsToSelectedClicked()
{
	QList<QListWidgetItem *> toAdd = applications->selectedItems();

	int count = toAdd.count();

	for (int i = 0; i < count; i++)
	{
		QString itemToAdd = toAdd[i]->data(Qt::UserRole).toString();

		bool exists = selectedContains(itemToAdd);
		
		if (appendedItems.indexOf(itemToAdd) == -1 && !exists)
		{
			appendedItems.append(itemToAdd);
		}

		if (! selectedContains(itemToAdd))
		{
			Application * ap = new Application( itemToAdd );
			appendApplication( ap, selected);
			delete ap;
		}
		deletedItems.removeAll(itemToAdd);
	}
}

bool MimeConfig::selectedContains(QString application)
{
	bool exists = false;
	int scount = selected->count();
	for (int j = 0; j < scount; j++)
	{
		QString existent = selected->item(j)->data(Qt::UserRole).toString();
		if (existent == application)
			exists = true;
	}
	return exists;
}

void MimeConfig::upClicked()
{
	int row = selected->currentRow();

	if (row > 0)
	{
		QListWidgetItem * item = selected->takeItem(row);
		selected->insertItem(row - 1, item);
		selected->setCurrentRow( row - 1 );
	}
}

void MimeConfig::downClicked()
{
	int row = selected->currentRow();

	if (row != -1 && row < selected->count() - 1)
	{
		QListWidgetItem * item = selected->takeItem(row);
		selected->insertItem(row + 1, item);
		selected->setCurrentRow( row + 1 );
	}
}

void MimeConfig::applicationsChanged( int row )
{
	applicationsToSelected->setEnabled( row != -1 );
}

void MimeConfig::selectedChanged( int row )
{
	deleteButton->setEnabled( row != -1 );
	if ( row == -1 )
	{
		up->setEnabled(false);
		down->setEnabled(false);

	} else {
		up->setEnabled( row != 0 );
		down->setEnabled( row < selected->count() - 1 );
	}
}
