#include <QDir>

#include "dirwidget.h"
#include "filedialog.h"

DirWidget::DirWidget( QWidget * parent ) : QWidget(parent)
{
	search = 0;
	currentBrowserPage = -1;
	cut = false;

        QVBoxLayout * currentDirLayout = new QVBoxLayout(this);
	currentDirLayout->setContentsMargins(0, 0, 0, 0);

        currentDirLabel = new QLabel(this);
	QFont font;
	font.setBold(true);
	currentDirLabel->setFont(font);

	QString chroot;

	if ( Settings::getBooleanValue( "ChrootEnvironment" ) )
		chroot = CHROOTMOUNT;

#ifdef BASICFSMODE
    model = new BasicFileSystemModel();
#else
    model = new FileSystemModel( chroot );
#endif

	QStringList validParents;
	validParents += QDir::homePath();
	validParents += SHARED_DIR;
	validParents += SEARCH_DIR;

        currentDirView = new ListView(validParents, this);
        //currentDirView->setMovement(QListView::Snap);
        //currentDirView->setUniformItemSizes(true);
        currentDirView->setModel( model );
	currentDirView->setSortingEnabled( true );

        currentDirLayout->addWidget(currentDirLabel);
        currentDirLayout->addWidget(currentDirView);

	rightClickMenu = new DirWidgetMenu(this);
	fillMenu();

        setListView();

	connect(currentDirView, SIGNAL(open(const QModelIndex &)), this, SLOT(listViewClicked(const QModelIndex &)));
	connect(currentDirView, SIGNAL(rightClicked()), this, SLOT(listViewRightClicked()));
	connect(currentDirView, SIGNAL(rootIndexChanged (const QModelIndex &)), this, SLOT(setLabel(const QModelIndex &)));
	connect(this, SIGNAL(enablePaste(bool)), rightClickMenu->pasteAction(), SLOT(setEnabled(bool)));

	emit enablePaste(false);
}

DirWidget::~DirWidget()
{
	if (search != 0)
	{
		search->wait();
		delete search;
	}
}

void DirWidget::fillMenu()
{
	connect(rightClickMenu, SIGNAL(removeTriggered()), this, SLOT(removeTriggered()));
	connect(rightClickMenu, SIGNAL(renameTriggered()), this, SLOT(renameTriggered()));
	connect(rightClickMenu, SIGNAL(openWithTriggered()), this, SLOT(openWithTriggered()));
	connect(rightClickMenu, SIGNAL(cutTriggered()), this, SLOT(cutTriggered()));
	connect(rightClickMenu, SIGNAL(copyTriggered()), this, SLOT(copyTriggered()));
	connect(rightClickMenu, SIGNAL(pasteTriggered()), this, SLOT(pasteTriggered()));
	connect(rightClickMenu, SIGNAL(newFolderTriggered()), this, SLOT(newFolderTriggered()));
	connect(rightClickMenu, SIGNAL(propertiesTriggered()), this, SLOT(propertiesTriggered()));
	connect(rightClickMenu, SIGNAL(locationTriggered()), this, SLOT(locationTriggered()));
}

void DirWidget::goUp()
{
	QModelIndex index = currentDirView->rootIndex();
	if ( index.isValid() )
	{
		setRootIndex(index.parent());
	}
}

void DirWidget::goPrevious()
{
	if (currentBrowserPage > 0)
	{
		currentBrowserPage--;
	}
	setRootIndex(browserCache[currentBrowserPage]);
}

void DirWidget::goNext()
{
	if ( currentBrowserPage < browserCache.count() - 1 )
	{
		currentBrowserPage++;
		setRootIndex(browserCache[currentBrowserPage]);
	}
}

void DirWidget::listViewClicked(const QModelIndex & index)
{
#ifndef BASICFSMODE
    model->clearChecklist();
#endif
	fileInfo = model->fileInfo(index);

	qDebug("\nfileInfo: %s\n", fileInfo.absoluteFilePath().toUtf8().data());

	if (fileInfo.isDir())
	{
		if (fileInfo.isExecutable())
			setRootIndex(index);
		else
			QMessageBox::critical ( this, tr("Not enough permissions"), tr("You have no permission to browse this folder"));
	}

	if (fileInfo.isFile())
	{
		if (fileInfo.isReadable())
		{
			QString preferredProgram = Application::preferredProgram("." + fileInfo.suffix());
			if ( preferredProgram.isEmpty() )
			{
				MimeConfig * mimeConfig = new MimeConfig(fileInfo.absoluteFilePath(), this);
				connect(mimeConfig, SIGNAL(mimeChanged()), this, SLOT(setIconProvider()));
				if ( mimeConfig->exec() == QDialog::Accepted )
				{
					preferredProgram = Application::preferredProgram("." + fileInfo.suffix());
				}
			}
			if ( ! preferredProgram.isEmpty() )
			{
				emit openFile(fileInfo.absoluteFilePath(), QString::null);
			}
		} else {
			QMessageBox::critical ( this, tr("Not enough permissions"), tr("You have no permission to open this file"));
		}
	}
}

void DirWidget::listViewRightClicked()
{
	QModelIndex index = currentDirView->indexAt(currentDirView->getRightClickPos());

#ifndef BASICFSMODE
	if (! model->isChecked( index ) )
	{
		model->clearChecklist();
	}
#endif

	qDebug("Index clicked: %s", index.data().toString().toUtf8().data());

	if (index.isValid())
	{
		fileInfo = model->fileInfo(index);

		if (fileInfo.isDir())
			rightClickMenu->setDirMenu();

		if (fileInfo.isFile())
			rightClickMenu->setFileMenu();

		if (fileInfo.isSymLink())
			rightClickMenu->setLinkMenu();
	} else {
		fileInfo = QFileInfo();
		rightClickMenu->setOtherMenu();
	}

	rightClickMenu->popup(QCursor::pos());
}

void DirWidget::setIconView()
{
        currentDirView->setIconSize(QSize(ICON_SIZE, ICON_SIZE));
        //currentDirView->setViewMode(QListView::IconMode);
        //currentDirView->setSpacing(10);
}

void DirWidget::setListView()
{
        currentDirView->setIconSize(QSize(ICON_SIZE / 2, ICON_SIZE / 2));
        //currentDirView->setViewMode(QListView::ListMode);
        //currentDirView->setSpacing(0);
}

void DirWidget::setHome()
{
	setRootIndex(model->setRootPath(QDir::homePath()));
	currentDirView->repaint();
	emit atHome();
}

void DirWidget::setShared()
{
	setRootIndex(model->setRootPath(SHARED_DIR));
	emit atShared();
}

void DirWidget::setSearchs()
{
	setRootIndex(model->setRootPath(Settings::searchPath()));
	emit atSearchs();
}

void DirWidget::setDropbox()
{
	setRootIndex(model->setRootPath(QDir::homePath() + QString(QDir::separator()) + DROPBOX_HOME ));
	emit atDropbox();
}

void DirWidget::setGoogleDrive()
{
	setRootIndex(model->setRootPath(QDir::homePath() + QString(QDir::separator()) + GOOGLEDRIVE_HOME ));
	emit atGoogleDrive();
}

void DirWidget::setDir(QString dir)
{
	setRootIndex(model->setRootPath(dir));
	qDebug("Entering at directory %s", dir.toUtf8().data());
}

void DirWidget::setLabel(const QModelIndex & index)
{

	QString newPath = model->filePath(index);

	correctPath(&newPath);

	currentDirLabel->setText(newPath);

	if (newPath.startsWith( tr("Home Folder") ))
		emit atHome();
	if (newPath.startsWith( tr("Shared Folder") ))
                emit atShared();
	if (newPath.startsWith( tr("Searchs Folder") ))
                emit atSearchs();
	if (newPath.startsWith( tr("Dropbox Folder") ))
                emit atDropbox();
	if (newPath.startsWith( tr("Google Drive Folder") ))
                emit atGoogleDrive();
}

void DirWidget::correctPath(QString * newPath)
{
	if ( ! Settings::searchPath().isEmpty())
		newPath->replace( QRegExp("^" + Settings::searchPath() ), tr("Searchs Folder") );
	if ( ! QDir::homePath().isEmpty())
		newPath->replace( QRegExp("^" + QDir::homePath() ), tr("Home Folder") );
	newPath->replace( QRegExp("^" SHARED_DIR ), tr("Shared Folder") );
	if ( ! QDir::homePath().isEmpty())
	{
		newPath->replace( QRegExp("^" + tr("Home Folder") + QString(QDir::separator()) + DROPBOX_HOME ), tr("Dropbox Folder") );
		newPath->replace( QRegExp("^" + tr("Home Folder") + QString(QDir::separator()) + GOOGLEDRIVE_HOME ), tr("Google Drive Folder") );
	}
	newPath->replace('/', " > ");
}

void DirWidget::refresh()
{
	setCursor(Qt::WaitCursor);
#ifndef BASICFSMODE
    model->update( currentIndex );
#endif
	setRootIndex( currentIndex );
	currentDirView->update( currentIndex );
	setCursor(Qt::ArrowCursor);
}

void DirWidget::setRootIndex(const QModelIndex & index)
{
	setCursor(Qt::WaitCursor);

	int count = browserCache.count();

	if ( currentIndex.isValid() )
	{
		if ( currentBrowserPage < count )
		{
			if ( currentIndex == browserCache[currentBrowserPage] ) // Not pressed 'next' or 'previous'
			{
				for ( int i = count - 1; i > currentBrowserPage; i-- )
				{
					browserCache.removeAt( i );
					count = i;
				}
				browserCache.append( index );
				currentBrowserPage++;
			}
		}
	} else { // First index set
		browserCache.append( index );
		currentBrowserPage++;
	}

	currentDirView->setRootIndex(index);
	currentIndex = index;

	emit enablePrevious(currentBrowserPage > 0);
	emit enableNext(currentBrowserPage < count - 1);
	QModelIndex parentIndex = index.parent();
	if ( parentIndex.isValid() )
	{
		QString parent = model->fileInfo(parentIndex).absoluteFilePath();
		emit enableUp(currentDirView->isValidPath(parent));
	}

	setCursor(Qt::ArrowCursor);
}

void DirWidget::removeTriggered()
{
	QList<QFileInfo> fileInfoList;

#ifndef BASICFSMODE
	QSet<QPersistentModelIndex> checkedIndexes = model->checkedIndexes();
	Settings::logInfo( QString("Checked indexes: %1").arg( checkedIndexes.count() ) );
	if ( checkedIndexes.count() > 1 )
	{
		QSetIterator<QPersistentModelIndex> i(checkedIndexes);
		while (i.hasNext())
		{
			QFileInfo fileInfo = model->fileInfo( i.next() );
			if ( !fileInfo.fileName().isEmpty() )
				fileInfoList.append( fileInfo );
		}
	} else {
		fileInfoList.append( fileInfo );
	}

	for ( int i = 0; i < fileInfoList.count(); i++ )
	{
		QString title;
		QString text;
		if (fileInfoList.at(i).isSymLink())
		{
			title = tr("Delete link");
			text = tr("This operation will delete the link '%1' permanently,\n"
				"but the original file will remain on its location.\n"
				"Are you sure you want to delete this link?").arg(fileInfoList.at(i).fileName());
		} else if (fileInfoList.at(i).isDir()) {
			title = tr("Delete folder");
			text = tr("This operation will delete the folder '%1' permanently\n"
				"with all its contents, folders and files.\n"
				"Are you sure you want to delete this folder?").arg(fileInfoList.at(i).fileName());
		} else {
			title = tr("Delete file");
			text = tr("This operation will delete the file '%1' permanently.\n"
				"Are you sure you want to delete this file?").arg(fileInfoList.at(i).fileName());
		}
		if (QMessageBox::question ( this, title, text, QMessageBox::Yes | QMessageBox::No ) == QMessageBox::Yes)
		{
			setCursor(Qt::WaitCursor);
			if (fileInfoList.at(i).isDir())
			{
				FileDialog * fileDialog = new FileDialog(this);
				connect( fileDialog, SIGNAL(finished()), this, SLOT(refresh()) );
				QApplication::processEvents();
				fileDialog->doRemove(fileInfoList.at(i).absoluteFilePath());
			} else {
				QDir dir (fileInfoList.at(i).absolutePath());
				dir.remove(fileInfoList.at(i).fileName());
				refresh();
			}
		}
	}
#endif
}

bool DirWidget::removeDir(const QString dirName)
{
	bool result = true;
	QDir dir(dirName);
 
	if (dir.exists(dirName))
	{
		Q_FOREACH(QFileInfo info, dir.entryInfoList(QDir::NoDotAndDotDot | QDir::System | QDir::Hidden  | QDir::AllDirs | QDir::Files, QDir::DirsFirst))
		{
			if (info.isDir())
			{
				result = removeDir(info.absoluteFilePath());
			} else {
				result = QFile::remove(info.absoluteFilePath());
			}
 
			if (!result)
			{
				return result;
			}
		}
		result = dir.rmdir(dirName);
	}
	refresh();
 
	return result;
}

void DirWidget::renameTriggered()
{
	QModelIndex index = currentDirView->indexAt(currentDirView->getRightClickPos());
	currentDirView->edit(index);
}

void DirWidget::openWithTriggered()
{
	openWithMenu = rightClickMenu->getOpenWithMenu();
	openWithMenu->clear();
	
	QString extension = "." + fileInfo.suffix();
	QList<Application*> appList;
	appList = Application::applicationList(extension);
	int count = appList.count();
	for (int i = 0; i < count; i++)
	{
		AppAction * action = new AppAction( openWithMenu->addAction ( appList[i]->getIcon(), Application::getText(appList[i]->getName()) ) );
		action->setApplication(appList[i]->getName());
		action->setFile(fileInfo.absoluteFilePath());
		connect(action, SIGNAL(openFile(QString, QString)), this, SIGNAL(openFile(QString, QString)));
	}

	Application::deleteList( appList );

	if (count > 0)
		openWithMenu->addSeparator();
	openWithMenu->addAction ( QIcon(ICON_DIR_64 + QString(QDir::separator()) + "configure.png"), tr("Configure..."), this, SLOT(openMimeConfig()));
}

void DirWidget::openMimeConfig()
{
	MimeConfig * mimeConfig = new MimeConfig(fileInfo.absoluteFilePath(), this);
	mimeConfig->exec();
}

void DirWidget::cutTriggered()
{
	copyTriggered();
	cut = true;
}

void DirWidget::copyTriggered()
{
	clipboard.clear();
#ifndef BASICFSMODE
	QSet<QPersistentModelIndex> checkedIndexes = model->checkedIndexes();
	Settings::logInfo( QString("Checked indexes: %1").arg( checkedIndexes.count() ) );
	if ( checkedIndexes.count() > 1 )
	{
		QSetIterator<QPersistentModelIndex> i(checkedIndexes);
		while (i.hasNext())
		{
			QFileInfo fileInfo = model->fileInfo( i.next() );
			if ( ! fileInfo.fileName().isEmpty() )
				clipboard.append( fileInfo );
		}
	} else {
		clipboard.append( fileInfo );
	}
#endif
	cut = false;
	emit enablePaste(true);
}

void DirWidget::pasteTriggered()
{
	QString path;

	if (fileInfo.isDir()) // paste in this dir
	{
		path = fileInfo.absoluteFilePath();
	} else { // paste in current folder
		QModelIndex index = currentDirView->rootIndex();
		path = model->filePath(index);
	}

	for ( int i = 0; i < clipboard.count(); i++ )
	{
		if ( clipboard.at(i).exists() )
		{
			setCursor(Qt::WaitCursor);
			FileDialog * fileDialog = new FileDialog();
			connect( fileDialog, SIGNAL(finished()), this, SLOT(refresh()) );
			fileDialog->showFullScreen();
			QApplication::processEvents();
			if (cut)
			{
				fileDialog->doMove(clipboard.at(i).absoluteFilePath(), path);
			} else {
				fileDialog->doCopy(clipboard.at(i).absoluteFilePath(), path);
			}
		}
	}
	emit enablePaste(false);
}

void DirWidget::newFolderTriggered()
{
	bool ok;

	QString res = QInputDialog::getText ( this, tr("Create new folder"),
						tr("Please, enter the name of the new folder"),
						QLineEdit::Normal,
						QString(),
						&ok);

	setCursor(Qt::WaitCursor);

	if ( ! res.isEmpty() )
	{
		QString path;

		if (fileInfo.isDir()) // paste in this dir
		{
			path = fileInfo.absoluteFilePath();
		} else { // paste in current folder
			QModelIndex index = currentDirView->rootIndex();
			path = model->filePath(index);
		}

		QString dir = path + "/" + res;

		if (QFile::exists( dir ) )
		{
			QMessageBox::critical ( this, tr("Cannot create folder"), tr("The folder name currently exists"));
			return;
		}

		if ( ! model->fileInfo(currentDirView->rootIndex()).isWritable() )
		{
			QMessageBox::critical ( this, tr("Cannot create folder"), tr("You have no permission to create this folder here"));
			return;
		}

		QString program = "mkdir";

		QStringList arguments;

		arguments << dir;
	
		if ( QProcess::execute( program, arguments ) )
		{
			QMessageBox::critical ( this, tr("Cannot create dir"), tr("There was an error creating the folder"));
		}

//		QModelIndex index = model->index(path);
//		model->mkdir(index, res);
	}
	refresh();
}


void DirWidget::propertiesTriggered()
{
	FileProperties * fileProperties = new FileProperties(fileInfo, model->fileIcon(model->index(fileInfo.absoluteFilePath())), this);
	fileProperties->exec();
}

void DirWidget::locationTriggered()
{
	QFileInfo target = QFileInfo(fileInfo.symLinkTarget());

	QString path = target.absolutePath();

	listViewClicked( model->index(path) );
}

void DirWidget::searchFiles(QString exp)
{
	/* First, remove searchs directory contents */
	QDir searchsDir(Settings::searchPath());
	QStringList entries = searchsDir.entryList();
	int numEntries = entries.count();
	for (int i=0; i<numEntries; i++)
	{
		searchsDir.remove(entries[i]);
	}

	/* Find files */
	QStringList dirs;
	dirs += SHARED_DIR;
	dirs += QDir::homePath();

	if (search != 0)
		delete search;

	search = new Search(exp, dirs);
	connect(search, SIGNAL(searchDone()), this, SLOT(searchDone()));
	search->start();
}

void DirWidget::searchDone()
{
	if (search != 0)
	{
		QFileInfoList files = search->foundFiles();

		search->wait();
		delete search;
		search = 0;

		int count = files.count();

		for (int i=0; i<count; i++)
		{
			QFile::link(files[i].absoluteFilePath(), Settings::searchPath() + "/" + files[i].fileName() );
		}

		setSearchs();
	}
	emit searchFinished();
}
