#include "filesystemmodel.h"

FileSystemModel::FileSystemModel(QString chroot, bool showHiddenFiles) : QStandardItemModel()
{
	this->chroot = chroot;
	this->showHiddenFiles = showHiddenFiles;

	fileIconProvider = new FileIconProvider();

	QIcon icon = fileIconProvider->icon( QFileInfo( chroot + "/" ) );
	rootItem = new QStandardItem( icon, "/");
	invisibleRootItem()->appendRow( rootItem );

	setHorizontalHeaderLabels( QStringList( tr("File name") ) );
}

FileSystemModel::~FileSystemModel()
{
	delete fileIconProvider;
}

QModelIndex FileSystemModel::parent( const QModelIndex & child ) const
{
	return QStandardItemModel::parent( child );

	if ( child.isValid() )
	{
		return child.parent();
	} else {
		return QModelIndex();
	}
}

QFileInfo FileSystemModel::fileInfo( const QModelIndex & index ) const
{
	QString filePath = this->filePath( index );
	return QFileInfo( filePath );
}

QVariant FileSystemModel::data(const QModelIndex & index, int role) const
{
	if ( role == Qt::CheckStateRole  && index.column() == 0 )
	{
		return checklist.contains(index) ? Qt::Checked : Qt::Unchecked;
	} else {
		return QStandardItemModel::data( index, role );
	}

	if ( ! index.isValid() )
		return QVariant();

	qDebug("Getting data from index %s, role %d", index.data().toString().toUtf8().data(), role);
	update( itemFromIndex( index ) );

	switch( role )
	{
		case Qt::DisplayRole:
			qDebug("Display: %s", itemFromIndex( index )->text().toUtf8().data());
			return itemFromIndex( index )->text();
		case Qt::TextAlignmentRole:
			return Qt::AlignVCenter;
		case Qt::CheckStateRole:
			if (index.column() == 0)
				return checklist.contains(index) ? Qt::Checked : Qt::Unchecked;
			else
				return QStandardItemModel::data(index, role);
		case FilePathRole:
		case FileNameRole:
		default:
			return QStandardItemModel::data(index, role);
	}
}
 
Qt::ItemFlags FileSystemModel::flags(const QModelIndex& index) const
{
	return QStandardItemModel::flags(index) | Qt::ItemIsUserCheckable;
}
 
bool FileSystemModel::setData(const QModelIndex& index, const QVariant& value, int role)
{
	//return QStandardItemModel::setData( index, value, role );
	switch( role )
	{
		case Qt::CheckStateRole:
			if (value == Qt::Checked)
				checklist.insert(index);
			else
				checklist.remove(index);
			emit dataChanged(index, index);
			return true;
		default:
			return QStandardItemModel::setData(index, value, role);
	}
}

bool FileSystemModel::isChecked(const QModelIndex& index)
{
	return checklist.contains( QPersistentModelIndex( index ) );
}

QSet<QPersistentModelIndex> FileSystemModel::checkedIndexes()
{
	return checklist;
}

void FileSystemModel::clearChecklist()
{
	checklist.clear();
}

QModelIndex FileSystemModel::setRootPath( const QString & newPath ) const
{
	Settings::logInfo( QString("Setting path to %1").arg(newPath.toUtf8().data()) );

	if ( QFile::exists( chroot + newPath ) )
	{
		QString p = "/";
		int depth = 0;
		QStringList pathList = newPath.split("/");
		QStandardItem * item = rootItem;

		while ( p != newPath )
		{
			qDebug(p.toUtf8().data());
			QString name = pathList[++depth];
			if ( p == "/" )	
				p += name;
			else
				p += "/" + name;

			QStandardItem * selectedItem = findNameInItemChildren(name, item);
			if ( selectedItem == 0 ) // not found
			{
				if ( item == rootItem )
					qDebug("Updating root (%d children). Hopefully only once.", item->rowCount());
				update( item );
				selectedItem = findNameInItemChildren(name, item);
				if ( selectedItem == 0 ) // not found, again, after update
				{
					return QModelIndex();
				}
			}
			item = selectedItem;
		}
		qDebug("Item %d for path %s", (int) item->index().internalId(), p.toUtf8().data());
		update( item );
		return item->index();
	} else {
		return QModelIndex();
	}
}

void FileSystemModel::update(QStandardItem * item) const
{
	QString path = chroot + "/" + filePath( item );
	path.replace( "//", "/" );

	qDebug("Updating path: %s", path.toUtf8().data() );

	item->removeRows( 0, item->rowCount() );

//	QStringList entries;

	QDir dir(path);

	if (dir.exists())
	{
		QStringList files = dir.entryList();

		for (int i = 0; i < files.count(); i++)
		{
			if ( files[i] != "." && files[i] != ".." )
			{
				if ( showHiddenFiles || ! files[i].startsWith( "." ) )
				{
					//entries += entry;
					//qDebug("Adding %s", entry.toUtf8().data());
					QIcon icon;

					icon = fileIconProvider->icon( QFileInfo( path + "/" + files[i] ) );

					QStandardItem * child = new QStandardItem( icon, files[i] );
					child->setCheckable( true );
					item->appendRow(child);
				}
			}
		}
	}
/*

	QStringList existent;

	for (int i = 0; i < item->rowCount(); i++)
	{
		QStandardItem * child = item->child(i);
		qDebug("child->text(): %s",  child->text().toUtf8().data());
		if ( ! entries.contains( child->text() ) )
		{
			remove( child );
		} else {
			existent.append(  child->text() );
		}
	}

	QStringListIterator it( entries );

	while ( it.hasNext() )
	{
		QString name = it.next();

		if ( ! existent.contains( name ) )
		{
			qDebug("Adding %s", name.toUtf8().data());
			QIcon icon;

			icon = fileIconProvider->icon( QFileInfo( path + "/" + name ) );

			QStandardItem * child = new QStandardItem( icon, name );
			child->setCheckable( true );
			item->appendRow(child);
		}
	}
*/
}

void FileSystemModel::remove(QStandardItem * item) const
{
	if ( item->hasChildren() )
	{
		int itemCount = item->rowCount();
		for (int i = 0; i < itemCount; i++)
		{
			remove( item->child(i) );
		}
	} else {
		delete item;
	}
}

QStandardItem * FileSystemModel::findNameInItemChildren(QString name, QStandardItem * item) const
{
	QStandardItem * selectedItem = 0;
	int itemCount = item->rowCount();
	qDebug("Item count: %d", itemCount);
	for (int i = 0; i < itemCount; i++)
	{
		if ( item->child(i)->text() == name )
		{
			selectedItem = item->child(i);
		}
	}
	return selectedItem;
}

QString FileSystemModel::filePath( QStandardItem * item ) const
{
	QStandardItem * iterator = item;

	QString path;

	if ( item == rootItem )
		qDebug( "Root item!" );

	while ( iterator != rootItem  && iterator != 0)
	{
		qDebug("Iterator: %s", iterator->text().toUtf8().data());
		if ( iterator == item )
			path = iterator->text();
		else
			path = iterator->text() + "/" + path;
		iterator = iterator->parent();
	}

	path = "/" + path;

	qDebug("filePath: %s", path.toUtf8().data());

	return path;
}

void FileSystemModel::setRootIndex(  const QModelIndex & index ) const
{
	setRootPath( filePath( index ) );
}

QString FileSystemModel::filePath( const QModelIndex & index ) const
{
	QStandardItem * item = itemFromIndex( index );
	return filePath( item );
}

QModelIndex FileSystemModel::index ( const QString & path ) const
{
    return QModelIndex();
    //return QFileSystemModel::index( path, 0 );
}
