#include <QDir>
#include <QWebView>
#include <QWebElement>
#include <QWebElementCollection>
#include <QWebFrame>

#include "googledriveconfig.h"
#include "googleauthdialog.h"

GoogleDriveConfig::GoogleDriveConfig(Menu * menu, GoogleDriveService * googleDriveService) : QDialog()
{
	this->googleDriveService = googleDriveService;
	connect( googleDriveService, SIGNAL(isUpdating( bool )), this, SLOT(refreshSlot()) );
	dontclose = true;
	setMenuWidget( menu );
	init ();

	refreshSlot();
}

void GoogleDriveConfig::init()
{
	layout = new QGridLayout(this);
	layout->setSpacing(10);
	layout->setMargin(20);

	backButton = new ToolButton(QIcon(ICON_DIR_64 + QString(QDir::separator()) + "back.png"), tr("Go back"), this);

	QLabel * titleLabel = new QLabel(tr("Google Drive Configuration"), this);
	QFont font;
	font.setBold(true);
	font.setPointSize(20);
	titleLabel->setFont( font );
	titleLabel->setAlignment( Qt::AlignVCenter );
	QLabel * titlePixmap = new QLabel( this );
	titlePixmap->setPixmap( QPixmap( ICON_DIR_APPS + QString(QDir::separator()) + "google-drive.png" ) );

	model = new GoogleDriveManager(googleDriveService);

	drives = new QTreeView( this );
	drives->setModel( model );
	drives->setHeaderHidden( true );
	font.setPointSize(16);
	font.setBold(false);
	drives->setFont( font );
	drives->setIconSize( QSize(MENU_ICON_SIZE, MENU_ICON_SIZE) );
	drives->expandAll();
	connect( drives, SIGNAL(clicked( const QModelIndex & )), this, SLOT(itemChanged( const QModelIndex & )) );

	refresh = new ToolButton(QIcon(ICON_DIR_64 + QString(QDir::separator()) + "refresh.png"), tr("Refresh"), this);
	add = new ToolButton(QIcon(ICON_DIR_64 + QString(QDir::separator()) + "add.png"), tr("Add"), this);
	remove = new ToolButton(QIcon(ICON_DIR_64 + QString(QDir::separator()) + "remove.png"), tr("Remove"), this);

	remove->setEnabled( false );

	layout->addWidget(backButton, 0, 0);
	layout->addWidget(titlePixmap, 1, 0);
	layout->addWidget(titleLabel, 1, 1, 1, 2);
	layout->addWidget(drives, 2, 0, 6, 8);
	layout->addWidget(refresh, 8, 5);
	layout->addWidget(add, 8, 6);
	layout->addWidget(remove, 8, 7);

	for (int i = 0; i < 8; i++)
	{
		layout->setRowStretch(i, 1);
		layout->setColumnStretch(i, 1);
	}

	connect(backButton, SIGNAL(clicked()), this, SLOT(accept()));
	connect( add, SIGNAL(clicked()), this, SLOT(addSlot()) );
	connect( remove, SIGNAL(clicked()), this, SLOT(removeSlot()) );
	connect( refresh, SIGNAL(clicked()), this, SLOT(refreshSlot()) );
}

void GoogleDriveConfig::accept()
{
	if ( dontclose )
	{
		hide();
	} else {
		QDialog::accept();
	}
}

void GoogleDriveConfig::reject()
{
	if ( dontclose )
	{
		hide();
	} else {
		QDialog::reject();
	}
}

void GoogleDriveConfig::itemChanged( const QModelIndex & index )
{
	if (index.isValid())
	{
		if (index.parent() == QModelIndex()) // No parent, this is an account
		{
			remove->setEnabled( true );
		} else {
			remove->setEnabled( false );
		}
	} else {
		remove->setEnabled( false );
	}
}

void GoogleDriveConfig::refreshSlot()
{
	model->update();
}

void GoogleDriveConfig::removeSlot()
{
	QString account = drives->currentIndex().data().toString();

	if ( ! account.isEmpty() )
	{
		QMessageBox::StandardButton res = QMessageBox::question ( this,
				tr("Removing Google Drive account"),
				tr("You are about to remove Google Drive access to this account from LibreWork. Are you sure you want to do this?"),
				QMessageBox::Yes | QMessageBox::No,
				QMessageBox::No);
		if ( res == QMessageBox::Yes )
		{
			model->removeAccount( account );
			refreshSlot();
		}
	}
}

void GoogleDriveConfig::addSlot()
{
	QString account = QInputDialog::getText ( this, tr("Add Google Drive account"), tr("Please, insert the name of the account"), QLineEdit::Normal);
	if ( ! account.isNull() )
	{
		if ( model->getAccounts().contains( account ) )
		{
			QMessageBox::critical ( this, tr("Cannot add acount"), tr("The name you are trying to assign currently exists"));
		} else {
			Settings::logInfo( QString( "Adding Google Drive account '%1'" ).arg(account) );
			QUrl url = model->addAccount( account );
			qDebug("URL: %s", url.toString().toUtf8().data());
			Settings::logInfo( QString( "URL: %1" ).arg(url.toString()) );
			if ( ! url.isEmpty() )
			{
				GoogleAuthDialog * authDialog = new GoogleAuthDialog( url );
				authDialog->exec();
				QString code = authDialog->getCode();
				if (! code.isEmpty() )
				{
					Settings::logInfo( QString( "Adding Google Drive code '%1'" ).arg(code) );
					model->finishAddAccount( code );
				} else {
					Settings::logError( QString( "Cannot add Google Drive account '%1'" ).arg(account) );
				}
				delete authDialog;
			}
			refreshSlot();
		}
	}
}
