#include <QDir>
#include <QUrl>
#include <QWebView>
#include <QWebElement>
#include <QWebElementCollection>
#include <QWebFrame>
#include <QEvent>

#include "googleprintconfig.h"
#include "googleauthdialog.h"

GooglePrintConfig::GooglePrintConfig(Menu * menu) : QDialog()
{
	firstTime = true;
	dontclose = true;
	setMenuWidget( menu );
	init ();
}

GooglePrintConfig::~GooglePrintConfig()
{
	delete model;
}

void GooglePrintConfig::init()
{
	layout = new QGridLayout(this);
	layout->setSpacing(10);
	layout->setMargin(20);

	backButton = new ToolButton(QIcon(ICON_DIR_64 + QString(QDir::separator()) + "back.png"), tr("Go back"), this);

	QLabel * titleLabel = new QLabel(tr("Google Print Configuration"), this);
	QFont font;
	font.setBold(true);
	font.setPointSize(20);
	titleLabel->setFont( font );
	titleLabel->setAlignment( Qt::AlignVCenter );
	QLabel * titlePixmap = new QLabel( this );
	titlePixmap->setPixmap( QPixmap( ICON_DIR_APPS + QString(QDir::separator()) + "google-print.png" ) );

	model = new GooglePrintManager();

	printers = new QTreeView( this );
	printers->setModel( model );
	printers->setHeaderHidden( true );
	font.setPointSize(16);
	font.setBold(false);
	printers->setFont( font );
	printers->setIconSize( QSize(MENU_ICON_SIZE, MENU_ICON_SIZE) );
	printers->expandAll();
	connect( printers, SIGNAL(clicked( const QModelIndex & )), this, SLOT(itemChanged( const QModelIndex & )) );

	refresh = new ToolButton(QIcon(ICON_DIR_64 + QString(QDir::separator()) + "refresh.png"), tr("Refresh"), this);
	add = new ToolButton(QIcon(ICON_DIR_64 + QString(QDir::separator()) + "add.png"), tr("Add"), this);
	remove = new ToolButton(QIcon(ICON_DIR_64 + QString(QDir::separator()) + "remove.png"), tr("Remove"), this);
	conn = new ToolButton(QIcon(ICON_DIR_64 + QString(QDir::separator()) + "connect.png"), tr("Connect"), this);

	layout->addWidget(backButton, 0, 0);
	layout->addWidget(titlePixmap, 1, 0);
	layout->addWidget(titleLabel, 1, 1, 1, 2);
	layout->addWidget(printers, 2, 0, 6, 8);
	layout->addWidget(refresh, 8, 4);
	layout->addWidget(add, 8, 5);
	layout->addWidget(remove, 8, 6);
	layout->addWidget(conn, 8, 7);

	for (int i = 0; i < 8; i++)
	{
		layout->setRowStretch(i, 1);
		layout->setColumnStretch(i, 1);
	}

	connect( backButton, SIGNAL(clicked()), this, SLOT(accept()) );
	connect( add, SIGNAL(clicked()), this, SLOT(addSlot()) );
	connect( remove, SIGNAL(clicked()), this, SLOT(removeSlot()) );
	connect( conn, SIGNAL(clicked()), this, SLOT(connectSlot()) );
	connect( refresh, SIGNAL(clicked()), this, SLOT(refreshSlot()) );
}

void GooglePrintConfig::enterEvent ( QEvent * )
{
	if (firstTime)
	{
		firstTime = false;
		refreshSlot();
	}
}

void GooglePrintConfig::itemChanged( const QModelIndex & index )
{
	if (index.isValid())
	{
		if (index.parent() == QModelIndex()) // No parent, this is an account
		{
			remove->setEnabled( true );
			conn->setEnabled( false );
		} else { // This is a printer
			remove->setEnabled( false );
			conn->setEnabled( true );
		}
	} else {
		remove->setEnabled( false );
		conn->setEnabled( false );
	}
}

void GooglePrintConfig::refreshSlot()
{
	remove->setEnabled( false );
	conn->setEnabled( false );
	setCursor(Qt::WaitCursor);
	model->update();
	setCursor(Qt::ArrowCursor);
	printers->expandAll();
}

void GooglePrintConfig::addSlot()
{
	QString account = QInputDialog::getText ( this, tr("Add Google Print account"), tr("Please, insert the name of the account"), QLineEdit::Normal);
	if ( ! account.isNull() )
	{
		Settings::logInfo( QString( "Adding Google Print account '%1'" ).arg(account) );
		QUrl url = model->addAccount( account );
		Settings::logInfo( QString( "URL: %1" ).arg(url.toString()) );
		if ( ! url.isEmpty() )
		{
			GoogleAuthDialog * authDialog = new GoogleAuthDialog( url );
			authDialog->exec();
			QString code = authDialog->getCode();
			if (! code.isEmpty() )
			{
				Settings::logInfo( QString( "Adding Google Drive account with code '%1'" ).arg(code) );
				model->finishAddAccount( code );
			} else {
				Settings::logError( QString( "Not adding Google Drive account for url '%1'" ).arg(url.toString()) );
			}
			delete authDialog;
		}
		refreshSlot();
	}
}

void GooglePrintConfig::removeSlot()
{
	QString account = printers->currentIndex().data().toString();
	model->removeAccount( account );
	refreshSlot();
}

void GooglePrintConfig::connectSlot()
{
	QModelIndex index = printers->currentIndex();
	QModelIndex parent;

	if (index.isValid())
        {
		parent = index.parent();
                if (parent == QModelIndex()) // No parent, this is an account
			return;
	} else {
		return;
	}

	QString printer = index.data().toString();
	QString account = parent.data().toString();
	bool connected = model->isConnected( printer );

	if (connected)
		model->disconnectPrinter( account, printer );
	else
		model->connectPrinter( account, printer );

	refreshSlot();
}

void GooglePrintConfig::accept()
{
	if ( dontclose )
	{
		hide();
	} else {
		QDialog::accept();
	}
}

void GooglePrintConfig::reject()
{
	if ( dontclose )
	{
		hide();
	} else {
		QDialog::reject();
	}
}
