#include "googleprintmanager.h"

#include <QDir>
#include <QUrl>

#include <cups/cups.h>

GooglePrintManager::GooglePrintManager( QObject * parent ) : QStandardItemModel( parent )
{
	process = 0;
	int port = Settings::getValue( "Server/LibreworkServerPort" ).toInt();
	QString address = Settings::getValue( "Server/LibreworkServerAddress" );

	tcpSocket = new QTcpSocket(this);

	tcpSocket->connectToHost( address, port, QIODevice::ReadWrite );

	if (tcpSocket->waitForConnected( TIMEOUT ))
	{
		qDebug("Connected");
		// Authentication
        QString message = "login?" + Settings::getFullUserName() + "&&" + Settings::getPassword() + "\n";
		tcpSocket->write(message.toUtf8().data());
		tcpSocket->waitForReadyRead();
		QString response = tcpSocket->readAll();
		QStringList accounts = getAccounts();
		if (accounts.count() > 0)
			prunePrinters();
	} else {
		qDebug("Something failed");
	}
}

GooglePrintManager::~GooglePrintManager()
{
	QString message = "quit\n";
	tcpSocket->write(message.toUtf8().data());
	tcpSocket->waitForReadyRead();
	QString response = tcpSocket->readAll();
	tcpSocket->close();
	delete tcpSocket;
}

void GooglePrintManager::update()
{
	this->clear();

	QStringList accounts = getAccounts();

	int numAccounts = accounts.count();

	for (int i = 0; i < numAccounts; i++)
	{
		QStandardItem * account = new QStandardItem( QIcon(ICON_DIR_APPS + QString(QDir::separator()) + "google-print.png"), accounts[i] );
		account->setEditable( false );
		appendRow( account );
		QStringList printers = getPrinters( accounts[i] );

		int numPrinters = printers.count();

		for (int j = 0; j < numPrinters; j++)
		{
			qDebug("Is connected %s?", printers[j].toUtf8().data());
			QStandardItem * printer;
			if (isConnected( printers[j] ) )
			{
				printer = new QStandardItem( QIcon(ICON_DIR_64 + QString(QDir::separator()) + "printer-connected.png"), printers[j] );
			} else {
				printer = new QStandardItem( QIcon(ICON_DIR_64 + QString(QDir::separator()) + "printer.png"), printers[j] );
			}
			printer->setEditable( false );
			account->appendRow( printer );
		}
	}
}

bool GooglePrintManager::isConnected( QString printer )
{
	QString sanitizedName = namedPrinter( printer );
	sanitizedName.replace(" ", "_").replace("\\", "_").replace("-", "_");

	qDebug("Sanitized name: %s", sanitizedName.toUtf8().data());

	QStringList printers = CupsManager::getPrinters();

	return printers.contains( sanitizedName );
}

QString GooglePrintManager::namedPrinter(QString printer)
{
	return Settings::getFullUserName() + "_GCP_" + printer;
}

void GooglePrintManager::connectPrinter( QString account, QString printer )
{
	QString message = "add_google_printer?" + 
			Settings::homePath() + QString(QDir::separator()) + GOOGLEPRINT_CONF + "&" +
			account + "&" +
			printer + "\n";

	tcpSocket->write( message.toUtf8().data());
	tcpSocket->waitForReadyRead();
	QByteArray response = tcpSocket->readAll();
}

void GooglePrintManager::disconnectPrinter( QString account, QString printer )
{
	QString message = "remove_google_printer?" + 
			Settings::homePath() + QString(QDir::separator()) + GOOGLEPRINT_CONF + "&" +
			account + "&" +
			printer + "\n";
			
	tcpSocket->write( message.toUtf8().data());
	tcpSocket->waitForReadyRead();
	QByteArray response = tcpSocket->readAll();
}

void GooglePrintManager::prunePrinters()
{
	QString message = "prune_google_printers?" + 
			Settings::homePath() + QString(QDir::separator()) + GOOGLEPRINT_CONF + "\n";

	tcpSocket->write( message.toUtf8().data());
	tcpSocket->waitForReadyRead();
	QByteArray response = tcpSocket->readAll();
}

QStringList GooglePrintManager::getAccounts()
{
	QString command = GOOGLEPRINT_MANAGER;
	QStringList arguments;
	arguments += Settings::homePath() + QString(QDir::separator()) + GOOGLEPRINT_CONF;
	arguments += "list";

	return getCommandOutput(command, arguments);
}

QStringList GooglePrintManager::getPrinters(QString account)
{
	QString command = GOOGLEPRINT_MANAGER;
	QStringList arguments;
	arguments += Settings::homePath() + QString(QDir::separator()) + GOOGLEPRINT_CONF;
	arguments += "printers";
	arguments += account;

	return getCommandOutput(command, arguments);
}

QUrl GooglePrintManager::addAccount(QString account)
{
	QUrl url;
	QString command = "python";
	QStringList arguments;
	arguments += "-u";
	arguments += GOOGLEPRINT_MANAGER;
	arguments += Settings::homePath() + QString(QDir::separator()) + GOOGLEPRINT_CONF;
	arguments += "add";
	arguments += account;

	process = new QProcess;
	process->start( command, arguments );

	if (process->waitForStarted())
	{
		if (process->waitForReadyRead())
		{
			while ( process->bytesAvailable() > 0)
			{
				QByteArray output = process->readLine();
				if ( output.startsWith("https://accounts.google.com") )
				{
					QString urlString = output;
					urlString = urlString.replace( "%3A", ":" );
					urlString = urlString.replace( "%2F", "/" );
					url = QUrl( urlString.trimmed() );
				}
			}
		}
	}

	return url;
}

void GooglePrintManager::finishAddAccount(QString key)
{
	if ( process != 0 )
	{
		if ( process->state() == QProcess::Running )
		{
			process->write( QString( key + "\n" ).toUtf8().data() );
			process->waitForFinished();
		}

		delete process;
		process = 0;
	}
}

void GooglePrintManager::removeAccount(QString account)
{
	QString command = GOOGLEPRINT_MANAGER;
	QStringList arguments;
	arguments += Settings::homePath() + QString(QDir::separator()) + GOOGLEPRINT_CONF;
	arguments += "remove";
	arguments += account;

	getCommandOutput(command, arguments);
}

QStringList GooglePrintManager::getCommandOutput(QString command, QStringList arguments)
{
	QStringList ret;

	QProcess * process = new QProcess;
	process->start(command, arguments);
	if (process->waitForStarted())
	{
		if (process->waitForFinished())
		{
			while (process->canReadLine())
			{
				ret += process->readLine().trimmed();
				qDebug("Line: %s", ret.last().toUtf8().data());
			}
		}
	}

	delete process;
	
	return ret;
}
