#include "settings.h"

#include <QDir>

static QHostAddress localAddress;
static QString userName;
static QString password;
static QMap<QString,QString> extensions;
static int windowManagerBehaviour = FULLSCREEN;
static QString _homePath = QString("");

Settings::Settings()
{
}

void Settings::init()
{
	QString behaviour = getValue("Desktop/WindowManagerBehaviour");

    if ( behaviour.toLower() == "standard" )
    {
        windowManagerBehaviour = STANDARD;
    } else {
        windowManagerBehaviour = FULLSCREEN;
    }
}

int Settings::getWindowManagerBehaviour()
{
    return windowManagerBehaviour;
}

QHostAddress Settings::getLocalAddress()
{
	return localAddress;
}

QString Settings::getMimetype(QString extension)
{
    if ( ! extension.isEmpty() )
    {
        return extensions.value( extension ).simplified();
    } else {
        return QString::null;
    }
}

void Settings::setExtensions()
{
    QFile extensionsFile(EXTENSIONS_FILE);
    if (extensionsFile.open(QIODevice::ReadOnly))
    {
       QTextStream in(&extensionsFile);
       while (!in.atEnd())
       {
           QString res = in.readLine();
           extensions.insert( res.section(":", 0, 0), res.section(":", 1, 1));
       }
       extensionsFile.close();
    }
}

QStringList Settings::getApplications( QString mimetype )
{
    return getValue( MIMETYPES_FILE, mimetype ).split(":");
}

QString Settings::getLocalOs()
{
    QString os;
#ifdef Q_OS_WIN
    os = "windows";
#endif
#ifdef Q_OS_LINUX
    os = "linux";
#endif
    return os;
}

bool Settings::isLocalhost(QString address)
{
    if ( address.compare("127.0.0.1") )
    {
        return true;
    }
    if ( address.compare("localhost"), Qt::CaseInsensitive )
    {
        return true;
    }
    if ( address.compare(QHostInfo::localHostName(), Qt::CaseInsensitive) )
    {
        return true;
    }
    if ( address.compare(QHostInfo::localHostName() + "." + QHostInfo::localDomainName(), Qt::CaseInsensitive) )
    {
        return true;
    }
    QList<QHostAddress> list = QNetworkInterface::allAddresses();
    for (int i = 0; i < list.count(); i++ )
    {
        if ( list[i].protocol() == QAbstractSocket::IPv4Protocol )
        {
            if ( QHostAddress(address) == list[i] )
            {
                return true;
            }
        }
    }
    return false;
}

void Settings::setLocalAddress(QHostAddress address)
{
	localAddress = address;
}

void Settings::setValue(QString file, QString key, QString value)
{
	QSettings settings(file, QSettings::IniFormat);
	settings.setValue( key, value);
}

QString Settings::getValue(QString primaryFile, QString secondaryFile, QString key)
{
    if ( key.isEmpty() )
    {
        return QString::null;
    }

	QString value;

	if ( QFile::exists(primaryFile) )
	{
		QSettings settings(primaryFile, QSettings::IniFormat);
		value = settings.value( key ).toString();
	}

	if (value.isEmpty())
	{
		QSettings settings(secondaryFile, QSettings::IniFormat);
		value = settings.value( key ).toString();
	}

	return value;
}

QString Settings::getApplicationValue( QString appName, QString key )
{
	return getValue( MIME_DIR + QString(QDir::separator()) + "applications", Settings::homePath()  + "/.desktux/mime/applications", appName + "/" + key );
}

QString Settings::getValue(QString key)
{
	return getValue(CONFIG_FILE, key);
}

QString Settings::getValue(QString file, QString key)
{
    if ( key.isEmpty() )
    {
        return QString::null;
    }

	QSettings settings(file, QSettings::IniFormat);
	QString value = settings.value( key ).toString();

	return value;
}

QStringList Settings::allKeys(QString file)
{
    QSettings settings(file, QSettings::IniFormat);
	QStringList res = settings.allKeys();

	return res;
}

QString Settings::sharedPath()
{
	return getValue("SharedPath");
}

QString Settings::searchPath()
{
	return SEARCH_DIR;
}

QString Settings::windowsSharedPath()
{
	return "\\\\" + getValue("SambaServer") + getValue("SambaSharedPath");
}

QString Settings::windowsHomePath()
{
	return "\\\\" + getValue("SambaServer") + "\\" + getUserName();
}

void Settings::setHomePath( QString homePath )
{
	_homePath = homePath;
}

QString Settings::homePath()
{
	if ( ! _homePath.isEmpty() )
	{
		return _homePath;
	}
	QString hpath = getValue("Server/HomePath");
	if (hpath.isEmpty())
	{
		hpath = QDir::homePath();
	}

	_homePath = hpath + QDir::separator() + getUserName();
	return _homePath;
}

int Settings::ldapServerPort()
{
	int value = getValue("LdapServerPort").toInt();

	if (value != 0)
		return value;
	else
		return 389;
}

bool Settings::getBooleanValue(QString key)
{
	QString value = getValue( key );
	bool ok;

	int number = value.toInt(&ok);

	if ( ok ) // is number
	{
		return ( number != 0 );
	} else {
		return ( value.toLower() == "true" );
	}
}

QStringList Settings::getApplicationsList()
{
    QSettings settings(APPLICATIONS_FILE, QSettings::IniFormat);
	return settings.childGroups();
}

QStringList Settings::getMimeList()
{
	QStringList res;
	QSettings settings(MIME_DIR + QString(QDir::separator()) + "types", QSettings::IniFormat);
	res = settings.allKeys();

	QSettings homeSettings(Settings::homePath()  + "/.desktux/mime/types", QSettings::IniFormat);
	res += homeSettings.allKeys();

	res.removeDuplicates();

	return res;
}

QStringList Settings::getIds(QString file)
{
    QSettings settings(file, QSettings::IniFormat);
	return settings.childGroups();
}

QStringList Settings::getValues(QString file, QString field)
{
    QStringList res;
    QStringList ids = getIds( file );
    for (int i = 0; i < ids.count(); i++)
    {
        res += getValue(file, ids[i] +"/" + field);
    }
    return res;
}

QString Settings::getUserName()
{
	if (userName.isEmpty())
	{
		QFile credentials( QDir::homePath()  + "/.credentials" );

		if (credentials.open(QIODevice::ReadOnly))
		{
			userName = credentials.readLine();
			userName.remove('\n');
		}
	}

	return userName;
}

QString Settings::getFullUserName()
{
	QString name = qgetenv("USER");
	if (name.isEmpty())
	{
		name = qgetenv("USERNAME");
	}

	if ( ! name.isEmpty() )
	{
		qDebug("Full username: %s", name.toUtf8().data());
		return name;
	} else {
		qDebug("Falling back to getUserName()");
		return getUserName();
	}
}

QString Settings::getPassword()
{
	if ( password.isEmpty() )
	{
		QFile credentials( Settings::homePath()  + "/.credentials" );
		if (credentials.open(QIODevice::ReadOnly))
		{
			credentials.readLine();
			password = credentials.readLine();
			password.remove('\n');
		}
	}

	return password;
}

void Settings::setUserName(QString user)
{
	userName = user;
}

void Settings::setPassword(QString pass)
{
	password = pass;
}

void Settings::log(QString type, QString message)
{
	QString logString = QDateTime::currentDateTime().toString() + " - " + type + ": " + message;
	QFile logFile(Settings::homePath() + "/.desktux.log");
	if (logFile.open(QIODevice::WriteOnly | QIODevice::Append))
	{
		QTextStream out(&logFile);

		out << logString;
		out << "\n";
		logFile.close();
	}

	qDebug( logString.toUtf8().data() );
}

void Settings::logInfo(QString message)
{
	log("Info", message);
}

void Settings::logError(QString message)
{
	log("Error", message);
}
