#!/usr/bin/env python
# -*- coding: utf-8 -*-

import stat
import os
import errno
import magic
import fuse

from filesystem import FileSystem
from fsstat import FSStat

from oauth2client.client import flow_from_clientsecrets
from oauth2client.file import Storage as CredentialStorage
from oauth2client.tools import run
from apiclient.discovery import build
from apiclient import errors
from apiclient.http import MediaFileUpload

import httplib2
import json
import logging
import re
import iso8601
import threading
from datetime import datetime
from cache import Cache

from logger import create_logger
_logger=create_logger('lwcloudfs.googledrive')

CREDENTIALS_FILE='credentials.json'

CLIENT_SECRETS_FILE='/usr/share/apps/librework-cloudfs/google_drive_client_secrets.json'
MISSING_CLIENT_SECRETS_MESSAGE='Error: Missing ' + CLIENT_SECRETS_FILE
SCOPE='https://www.googleapis.com/auth/drive'
CACHETTL=120

class GoogleDrive(FileSystem):
	def __init__( self ):
		FileSystem.__init__(self)
		self.cacheJsonData = Cache(CACHETTL)
		self.cacheId = Cache(CACHETTL)

	def init( self, credentialsFile ):
		if not os.path.isfile( CLIENT_SECRETS_FILE ):
			flow_from_clientsecrets( 'google_drive_client_secrets.json', scope=SCOPE, message=MISSING_CLIENT_SECRETS_MESSAGE)
		else:
			flow_from_clientsecrets(CLIENT_SECRETS_FILE, scope=SCOPE, message=MISSING_CLIENT_SECRETS_MESSAGE)
		credential_storage = CredentialStorage(credentialsFile)
		self.credentials = credential_storage.get()
		if self.credentials is None or self.credentials.invalid:
			print "Error: invalid credentials file"
			exit(4)
		# Get 'about' information in order to refresh access token
		try:
			self.mutex = threading.Lock()
			http = self.credentials.authorize(httplib2.Http())
			self.service = build('drive', 'v2', http=http)
			self.service.about().get().execute()
		except:
			pass


	def authorize( self, type, credentialsFile ):
		credentials = None
		if not os.path.isfile( CLIENT_SECRETS_FILE ):
			flow = flow_from_clientsecrets( 'google_drive_client_secrets.json', scope=SCOPE, message=MISSING_CLIENT_SECRETS_MESSAGE)
		else:
			flow = flow_from_clientsecrets(CLIENT_SECRETS_FILE, scope=SCOPE, message=MISSING_CLIENT_SECRETS_MESSAGE)
		credentialStorage = CredentialStorage(credentialsFile)
		if type == "console":
			flow.redirect_uri='urn:ietf:wg:oauth:2.0:oob'
			auth_uri = flow.step1_get_authorize_url()
			print("Open this URL accepting access to LibreWork CloudFS: \n\n" + auth_uri + "\n")
			code = raw_input('Code from Google: ')
			try:
				print("")
				credentials = flow.step2_exchange(code)
			except:
				print("\nError: The code does not seem to be valid.\n")
				exit(5)
			credentialStorage.put( credentials )

		if type == "browser":
			credentials = run(flow, credentialStorage)
			http = credentials.authorize(httplib2.Http())
			build('storage', 'v1beta1', http=http)

		if credentials is None or credentials.invalid:
			print "Error creating credentials"
			exit(6)
		else:
			print "Successfully created credentials file " + credentialsFile
			exit(0)

	def umount( self ):
		_logger.debug('umount()')

	def exists( self, path ):
		_id = self.retrieveId( path )
		if _id == None:
			return False
		try:
			with self.mutex:
				self.service.files().get(fileId=_id).execute()
			return True
		except errors.HttpError:
			return False

	def downloadFile( self, fileName, localFile ):
		_logger.debug('downloadFile(%s)', fileName)
		_id = self.retrieveId( fileName )
		if _id != None:
			#try:
			with self.mutex:
				fileMetaData = self.service.files().get(fileId=_id).execute()
			if fileMetaData.get('mimeType') == "application/vnd.google-apps.presentation":
				downloadUrl = fileMetaData.get('exportLinks')['application/pdf']
			else:
				downloadUrl = fileMetaData.get('downloadUrl')
			if downloadUrl:
				with self.mutex:
					resp, content = self.service._http.request(downloadUrl)
				if resp.status == 200:
					_file = open(localFile, 'wb')
					_file.write(content)
					_file.close()
				else:
					print 'An error occurred: %s' % resp
			#except errors.HttpError, error:
			#	print 'An error occurred: %s' % error
			#	_logger.debug('An error occurred: %s', error)
			#except:
			#	_logger.debug('An error occurred')

	def uploadFile( self, path, localFile ):
		_logger.debug('uploadFile(%s)', path)
		_id = self.retrieveId( path )
		if _id != None:
			#try:
				_logger.debug('uploadFile(%s): update', path)
				fileName = localFile
				with self.mutex:
					fileMetaData = self.service.files().get(fileId=_id).execute()
				mimeType = fileMetaData['mimeType'].encode('utf8')
				_logger.debug('uploadFile(%s): Mime type: %s', path, mimeType)
				_logger.debug('uploadFile(%s): mediaBody', path)
				mediaBody = MediaFileUpload( fileName, mimetype=mimeType, resumable=False )
				_logger.debug('uploadFile(%s): updatedFile', path)
				with self.mutex:
					self.service.files().update( fileId=_id, body=fileMetaData, newRevision=0, media_body=mediaBody).execute()
				_logger.debug('uploadFile(%s): finished', path)
			#except errors.HttpError, error:
			#	print 'An error occurred: %s' % error
			#	_logger.debug('An error occurred: %s', error)
			#except:
			#	_logger.debug('An error occurred')
		else:
				self.cacheJsonData.remove( path )
				self.cacheId.remove( path )
			#try:
				_logger.debug('uploadFile(%s): upload', path)
				fileName = localFile
				m = magic.open(magic.MAGIC_MIME)
				m.load()
				mimeType = m.file( fileName ).split(';')[0]
				if mimeType == None:
					mimeType = 'text/plain'
				_logger.debug('uploadFile(%s): Mime type: %s', path, mimeType)
				
				mediaBody = MediaFileUpload( fileName, mimetype=mimeType, resumable=False )
				_logger.debug('uploadFile(%s): updatedFile', path)
				fileMetaData = {
					'title': self.getFileName( path ),
					'description': self.getFileName( path ),
					'mimeType': mimeType
				}
				fileMetaData['parents'] = [{'id': self.getParentId( path )}]
				with self.mutex:
					self.service.files().insert( body=fileMetaData, media_body=mediaBody).execute()
				_logger.debug('uploadFile(%s): finished', path)

			#except errors.HttpError, error:
			#	print 'An error occurred: %s' % error
			#	_logger.debug('An error occurred: %s', error)
			#	raise
			#except:
			#	_logger.debug('An error occurred')
			#	raise


	def createDir( self, dirName ):
		_logger.debug('createDir(%s)', dirName)
		self.create( dirName, True )
		_logger.debug('createDir(%s) done.', dirName)

	def remove( self, path ):
		_logger.debug('remove(%s)', path)
		_id = self.retrieveId( path )
		self.cacheJsonData.remove( _id )
		self.cacheId.remove( path )
		if not _id == None:
			try:
				with self.mutex:
					ret = self.service.files().trash(fileId=_id).execute()
				return ret
			except errors.HttpError, error:
				_logger.debug('An error occurred: %s', error)

	def getStat( self, path ):
		_logger.debug('getStat(%s)', path)

		_id = self.retrieveId( path )

		if _id == None:
			st = -errno.ENOENT
		else:
			st = self.getStatFromId( _id )
		_logger.debug('getStat: finished')
		return st

	def getStatFromId( self, _id ):
		_logger.debug('getStat: Retrieving jsonData for id %s', _id)

		st = FSStat()
		st.st_uid = os.getuid()
		st.st_gid = os.getgid()

		jsonData = self.retrieveJsonData( _id )

		_logger.debug('getStat: jsonData retrieved')

		if jsonData == None:
			st.st_mode = stat.S_IFREG | 0644
			return st

		_logger.debug('getStat: jsonData not null')

		if jsonData['mimeType'] == 'application/vnd.google-apps.folder':
			_logger.debug('getStat: is Folder')
			if jsonData['shared']:
				st.st_mode = stat.S_IFDIR | 0775
			else:
				st.st_mode = stat.S_IFDIR | 0755
			st.st_nlink = 2
			st.st_size = 4096
		else:
			_logger.debug('getStat: is File')
			if jsonData['shared']:
				st.st_mode = stat.S_IFREG | 0664
			else:
				st.st_mode = stat.S_IFREG | 0644
			st.st_nlink = 1
			st.st_size = int( jsonData['quotaBytesUsed'] )
			if jsonData['mimeType'] == 'application/vnd.google-apps.presentation':
				st.st_size = 1024

		st.st_ctime = self.getSeconds( iso8601.parse_date( jsonData['createdDate'] ) )
		st.st_mtime = self.getSeconds( iso8601.parse_date( jsonData['modifiedDate'] ) )
		try:
			st.st_atime = self.getSeconds( iso8601.parse_date( jsonData['lastViewedByMeDate'] ) )
		except:
			st.st_atime = st.st_ctime
		return st

	def duplicates( self, l):
		return set([x for x in l if l.count(x) > 1])

	def getSeconds( self, dateTime ):
		dateFrom = iso8601.parse_date( "1970-01-01 00:00:00" )
		delta = dateTime - dateFrom
		return delta.days * 24 * 3600 + delta.seconds
			
	def getDirContents( self, dirName ):
		res = [ '.', '..' ]

		#if dirName == "/":
		#	res.append( ".trash" )
		try:
			files = self.retrieveAllFiles( dirName )
			for _file in files:
				data = json.loads(json.dumps(_file))
				_id = data['id'].encode('utf8')
				title = data['title'].encode('utf8')
				_logger.debug('Caching data for id %s', _id)
				self.cacheJsonData.set(_id, data )
				if dirName == "/":
					path = dirName + title
				else:
					path = dirName + "/" + title
				self.cacheId.set(path, _id)
				res.append( title )
		except:
			pass

		dup = self.duplicates( res )
		if len(dup) > 0:
			res = [x for x in res if x not in dup]
			for _file in dup:
				parent = self.retrieveId( dirName )
				result = self.searchElements( "'" + parent + "' in parents and title='" + _file + "' and trashed=false" )
				for fileJson in result:
					data = json.loads(json.dumps(fileJson))
					res.append( _file + " (id=" + data['id'].encode('utf8') + ")" )
		return res

	def statFs (self):
		st = fuse.StatVfs()
		st.f_bsize = 1
		st.f_frsize = 1
		with self.mutex:
			about = self.service.about().get().execute()
		total = int(about['quotaBytesTotal'])
		used = int(about['quotaBytesUsed'])
		st.f_blocks = total
		st.f_bfree = total - used
		st.f_bavail = total - used
		return st

	def getParentId( self, path ):
		pathList = path.split('/')
		del pathList[-1]
		parent = "/".join(pathList)
		_id = self.retrieveId( parent )
		return _id

	def retrieveId( self, path ):
		_logger.debug('retrieveId(%s)', path)
		_id=self.cacheId.get(path)
		if _id:
			_logger.debug('retrieveId(%s) Retrieving cached id', path)
			return _id
		
		pathList=path.split('/')
		parent='root'

		for item in pathList:
			_logger.debug('retrieveId Item: %s', item)
			if item != '':
				result = []
				_logger.debug('Before re.compile')
				if re.match(".+ \(id=.+\)", item):
					_logger.debug('Found regexp')
					itemTuple = item.rpartition( " (id=" )
					name = itemTuple[0]
					_id = itemTuple[2][:-1]
					jsonData = self.retrieveJsonData( _id )
					if not jsonData == None:
						data = json.loads(json.dumps(jsonData))
						if data['title'].encode('utf8') == name:
							result.append( jsonData )
				else:
					_logger.debug('Not found regexp, doing normal')
				if result == []:
					s = "'" + parent + "' in parents and title='"
					enc = item.decode('utf8')
					s += enc
					s += "' and trashed=false"
					result = self.searchElements( s.encode('utf8') )
				data={}
				_logger.debug('searchElements len: %d', len( result ) )
				if len( result ) > 0:
					fileJson = result[0]
					data = json.loads(json.dumps(fileJson))
					parent = data['id']
				else:
					_logger.debug('No parent for item %s', item)
					parent = None
					break
		_logger.debug('retrieveId(%s) Id: "%s"', path.decode('utf8'), parent)
		if parent != None:
			self.cacheId.set(path, parent)
		return parent

	def retrieveAllFiles( self, path ):
		""" Retrieve a list of File resources.
		Args:
			service: Drive API service instance.
			path: the parent dir
		Returns:
			List of File resources.
		"""
		_logger.debug('retrieveAllFiles(%s)', path)
		if path == "/":
			searchString = "'root' in parents and trashed=false"
		else:
			if path == "/.trash":
				searchString = "trashed=true"
			else:
				parent = self.retrieveId( path )
				searchString = "'" + parent + "' in parents and trashed=false"
		return self.searchElements( searchString )

	def searchElements( self, searchString ):
		""" Performs a search on Google Drive
		Args:
			searchString: the search string
		Returns:
			List of JSON strings with found elements
		"""
		result = []
		pageToken = None
		first = True
		while first or pageToken:
			try:
				first = False
				param = {}
				param['pageToken'] = pageToken
				param['q'] = searchString.decode('utf8')
				with self.mutex:
					files = self.service.files().list(**param).execute()

				result.extend(files['items'])
				pageToken = files.get('nextPageToken')
			except errors.HttpError, error:
				print 'An error occurred: %s' % error
				break
		return result

	def rename( self, oldPath, newPath ):
		_logger.debug('rename(%s to %s)', oldPath, newPath)
		_id = self.retrieveId( oldPath )
		if _id != None:
			try:
				_file = {'title': self.getFileName( newPath )}
				with self.mutex:
					self.service.files().patch( fileId=_id, body=_file, fields='title').execute()
				newId = self.cacheId.get( newPath )
				if newId != None:
					self.cacheId.remove(newPath)
					self.cacheJsonData.remove(newId)
				self.cacheId.remove(oldPath)
				self.cacheJsonData.remove(_id)
				return True
			except errors.HttpError, error:
				print 'An error occurred: %s' % error
				return False

	def create( self, path, isDir ):
		_tuple = path.rpartition( "/" )
		name = _tuple[2]
		parentPath = _tuple[0]
		parent = self.retrieveId( parentPath )
		if isDir:
			mime_type = 'application/vnd.google-apps.folder'
		else:
			mime_type = ''
		body = {
			'title': name,
			'mimeType': mime_type
		}

		# Set the parent folder.
		if parent:
			body['parents'] = [{'id': parent}]
		# Create path
		try:
			_logger.debug('service.files().insert() (%s)', path)
			with self.mutex:
				self.service.files().insert(body=body).execute()
			_logger.debug('insert done (%s)', path)
		except errors.HttpError, error:
			print 'An error occured: %s' % error

	def retrieveJsonData(self, _id):
		_logger.debug('retrieveJsonData(%s)', _id)
		jsonData=self.cacheJsonData.get(_id)
		if not jsonData:
			try:
				with self.mutex:
					jsonData = self.service.files().get(fileId=_id).execute()
				self.cacheJsonData.set( _id, jsonData )
			except (errors.HttpError, TypeError) as error:
				_logger.debug( 'retrieveJsonData: An error occurred: %s', error )
				jsonData = None
		return jsonData
