#!/usr/bin/env python

import os
import stat
import errno
import tempfile
import time
import threading
import shutil
from cache import Cache

import fuse
fuse.fuse_python_api = (0, 2)

from filesystem import FileSystem
from fsstat import FSStat
from googledrive import GoogleDrive
from dropbox import Dropbox

from logger import create_logger
_logger=create_logger('lwcloudfs.lwcloudfs')

DELAYEDUPLOADS = False	# set delayed uploads on
UPLOADDELAY = 20	# in seconds, time to start uploading

CACHETTL = 120		# in seconds, cache time to live

TMPPREFIX = 'lwcloudfs-'
TMPSUFFIX = '.tmp'

class Upload (threading.Thread):
	def __init__(self, path, localFile, lwCloudFs):
		threading.Thread.__init__(self)
		self.path = path
		self.localFile = localFile
		self.lwCloudFs = lwCloudFs
		self.status = "waiting"
		self.mutex=threading.Lock()
	def run(self):
		time.sleep( UPLOADDELAY )
		with self.mutex:
			self.status = "uploading"

		if os.path.exists( self.localFile ):
			self.lwCloudFs.fs.uploadFile( self.path )

		with self.mutex:
			self.status = "finished"
			if self.path in self.lwCloudFs.notUploaded:
				self.lwCloudFs.notUploaded.remove( self.path )
			self.lwCloudFs.fileWritten[self.path] = False

class LWCloudFS(fuse.Fuse):
	"""
	"""
	def __init__(self, *args, **kw):
		fuse.Fuse.__init__(self, *args, **kw)

		self.statCache = Cache( CACHETTL )
		self.dirCache = Cache( CACHETTL )

		self.fileWritten = {}		# fileWritten[path] = bool
		self.inodes = {}
		self.lastInode = 0
		self.notUploaded = []		# files created locally but not uploaded
		self.uploads = {}		# uploads[path] = Upload (threading.Thread)

		self.fileWrittenMutex = threading.RLock()
		self.inodesMutex = threading.RLock()
		self.notUploadedMutex = threading.RLock()
		self.tempFileMutex = threading.RLock()
		self.fsMutexMutex = threading.RLock()
		self.fsMutex = {}

		self.parser.add_option(	"--fstype",
					action="store",
					dest="fstype",
					default="googledrive",
					help="file system type (googledrive, dropbox, skydrive, mega) [default: %default]")
		self.parser.add_option(	"--tmpdir",
					action="store",
					dest="tmpdir",
					default="/tmp",
					help="temporary cache dir [default: %default]")
		self.parser.add_option(	"--credsfile",
					action="store",
					dest="credsfile",
					default="credentials.json",
					help="credentials file [default: %default]")
		self.parser.add_option(	"--authorize",
					action="store",
					dest="authorize",
					default="none",
					help="generate credentials file (console, browser, none) [default: %default]")
		self.parse(values=self, errex = 1)

		self.options, a = self.parser.parse_args()

		self.tmpDir = tempfile.mkdtemp( TMPSUFFIX, TMPPREFIX, self.options.tmpdir)

		if self.options.fstype == "googledrive":
			self.fs = GoogleDrive()
		elif self.options.fstype == "dropbox":
			self.fs = Dropbox()
		elif self.options.fstype == "mega":
			print "Not yet implemented fstype '" + self.options.fstype +"'"
			exit(3)
		elif self.options.fstype == "skydrive":
			print "Not yet implemented fstype '" + self.options.fstype +"'"
			exit(3)
		else:
			print "Unknown fstype value '" + self.options.fstype +"'"
			exit(3)

		if not self.options.authorize == "none":
			self.fs.authorize(self.options.authorize, self.options.credsfile)

		self.fs.init(self.options.credsfile)

	def getFsMutex( self, path ):
		with self.fsMutexMutex:
			if not path in self.fsMutex:
				self.fsMutex[path] = threading.RLock()
		return self.fsMutex[path]

	def removeFromDirCache( self, path ):
		with self.getFsMutex( path ):
			fileName = self.fs.getFileName( path )
		parent = self.getParent( path)

		dirList = self.dirCache.get( parent )

		if dirList != None:
			if fileName in dirList:
				self.dirCache.cache[parent][1].remove( fileName )

	def addToDirCache( self, path ):
		_logger.debug('Adding %s to dir cache', path)
		with self.getFsMutex( path ):
			fileName = self.fs.getFileName( path )
		parent = self.getParent( path)
		_logger.debug('Adding %s file to dir %s cache', fileName, parent)

		dirList = self.dirCache.get( parent )

		if dirList != None:
			if not fileName in dirList:
				self.dirCache.cache[parent][1].append( fileName )
				_logger.debug('Added %s file to dir %s cache', fileName, parent)

	def getParent( self, path ):
		pathList = path.split('/')
		l = len(pathList)
		newPath = ""
		for index,item in enumerate(pathList):
			if index > 0 and index < l - 1:
				newPath = newPath + "/" + item
		return newPath

	def hashFileName( self, path ):
		_hash = self.fs.retrieveNameHash( path )
		return self.tmpDir + "/" + _hash

	def tempFileName( self, path ):
		with self.getFsMutex( path ):
			_temp = self.fs.retrieveId( path )
		_logger.debug('*** tempFileName: %s', _temp)
		_logger.debug('*** self.tmpDir: %s', self.tmpDir)
		if _temp != None:
			return self.tmpDir + "/" + _temp
		else:
			return None

	def getTempFile( self, path ):
		hashFile = self.hashFileName( path )
		if os.path.isfile( hashFile ):
			return hashFile

		tempFile = self.tempFileName( path )
		if  tempFile != None:
			if os.path.isfile( tempFile ):
				return tempFile
		return None

	def getStatFromTemp( self, path ):
		with self.tempFileMutex:
			isDir = os.path.isdir( path )
			st = self.getBaseStat( isDir )
			if not isDir:
				st.st_size = os.stat( path ).st_size
			st.st_ctime = os.stat( path ).st_ctime
			st.st_mtime = os.stat( path ).st_mtime
			st.st_atime = os.stat( path ).st_atime
			return st

        def getBaseStat( self, isDir ):
		st = FSStat()
		st.st_uid = os.getuid()
		st.st_gid = os.getgid()
		if isDir:
			st.st_mode = stat.S_IFDIR | 0750
			st.st_nlink = 2
			st.st_size = 4096
		else:
			st.st_mode = stat.S_IFREG | 0640
			st.st_nlink = 1
		return st

	def setTimeNow( self, st ):
		st.st_ctime = int( time.time() )
		st.st_atime = int( time.time() )
		st.st_mtime = int( time.time() )

	def _access( self, path, flags ):
		return True

	def _exists( self, path ):
		if path == "/" or path == "." or path == "..":
			return True
		fstat = self.statCache.get( path )
		if fstat != None:
			if fstat == -errno.ENOENT:
				return False
			else:
				return True
		else:
			tempFile = self.getTempFile( path )
			if tempFile != None:
				return True
			with self.getFsMutex( path ):
				if self.fs.exists( path ):
					return True
			self.statCache.set( path, -errno.ENOENT )
			return False

	""" FUSE Filesystem functions """
	def fsdestroy(self):
		_logger.debug('*** fsdestroy')
		if DELAYEDUPLOADS:
			for upload in self.uploads:     # Wait until all uploads are finished
				if upload.isAlive():
					upload.join()
		self.fs.umount()
		shutil.rmtree( self.tmpDir )

	def access(self, path, flags):
		_logger.debug('*** access')
		if not self._exists(path):
			return -errno.ENOENT
		if flags != os.F_OK and not self._access(path, flags):
			return -errno.EACCES
		return 0

	def getattr(self, path):
		_logger.debug('*** getattr(%s)', path)
		with self.inodesMutex:
			if path in self.inodes:
				inode = self.inodes[path]
			else:
				self.lastInode = self.lastInode + 1
				inode = self.lastInode
				self.inodes[path] = inode

		st = self.statCache.get( path )
		if st != None:
			_logger.debug('*** getattr(%s) from cache', path)
			if st == -errno.ENOENT:
				_logger.debug( '-errno.ENOENT' )
			else:
				_logger.debug( st )
			return st

		tempFile = self.getTempFile( path )
		if tempFile != None:
			_logger.debug('*** getattr(%s) from temp', path)
			st = self.getStatFromTemp( tempFile )
		elif path == "/" or path == ".":
			st = self.getBaseStat( True )
			self.setTimeNow( st )
		elif path == "..":
			st = self.getStatFromTemp( path )
		else:
			_logger.debug('*** getattr(%s) from underlying filesystem', path)
			with self.getFsMutex( path ):
				st = self.fs.getStat( path )

		if st == -errno.ENOENT:
			_logger.debug( '-errno.ENOENT' )
		else:
			st.st_ino = inode
			st.st_blocks = 1 + st.st_size / 512
			_logger.debug( st )

		self.statCache.set( path, st )

		return st

	def chmod(self, path, mode):
		_logger.debug('*** chmod(%s, %d)', path, mode)
		return 0

	def chown(self, path, uid, gid):
		_logger.debug('*** chown(%s, %d, %d)', path, uid, gid)
		return 0

	def utime(self, path, times):
		_logger.debug('*** utime(%s)', path)
		atime, mtime = times

		tempFile = self.getTempFile( path )
		if tempFile != None:
			os.utime( tempFile, times )

		self.statCache.remove( path )
		st = self.getattr( path )

		if st != -errno.ENOENT:
			st.st_ctime = mtime
			st.st_atime = atime
			st.st_mtime = mtime

		self.statCache.set( path, st )

		if st != -errno.ENOENT:
			return 0
		else:
			return -errno.ENOENT

	def utimens(self, path, ts_acc, ts_mod):
		_logger.debug('*** utimens(%s)', path)
		atime = ts_acc.tv_sec + (ts_acc.tv_nsec / 1000000.0)
		mtime = ts_mod.tv_sec + (ts_mod.tv_nsec / 1000000.0)
		return self.utime( path, (atime, mtime) )

	def create(self, path, flags, mode):
		_logger.debug('*** create(%s)', path)
		mknod = self.mknod( path, mode, flags )
		if mknod == 0:
			return self.open( path, flags )
		else:
			return mknod

	def flush(self, path ):
		_logger.debug('*** flush(%s)', path)
		return 0

	def readdir(self, path, offset):
		rd = self.dirCache.get( path )
		if rd == None:
			_logger.debug('*** readdir(%s, %d)', path, offset)	   
			rd = self.fs.getDirContents( path )
			with self.notUploadedMutex:
				for x in self.notUploaded:
					rd.append( self.fs.getFileName( x ) )
			self.dirCache.set( path, rd )
		else:
			_logger.debug('*** readdir(%s, %d) cached', path, offset)

		return (fuse.Direntry(x) for x in rd)

	def mkdir ( self, path, mode ):
		_logger.debug('*** mkdir(%s, %d)', path, mode)
		with self.getFsMutex( path ):
			self.fs.createDir( path )
		self.statCache.remove( path )
		self.addToDirCache( path )

	def mknod ( self, path, mode, dev ):
		_logger.debug('*** mknod(%s, %d, %d)', path, mode, dev)
		if not self._exists( path ):
			hashFile = self.hashFileName( path )
			if not os.path.isfile( hashFile ):
				_file = open( hashFile, 'w')
				_file.close()
				with self.notUploadedMutex:
					if not path in self.notUploaded:
						self.notUploaded.append( path )
		self.addToDirCache( path )
		self.statCache.remove( path )
		_logger.debug('*** mknod(%s, %d, %d) finished', path, mode, dev)

	# This method could maintain opened (or locked) file list and,
	# of course, it could check file permissions.
	def open ( self, path, flags ):
		_logger.debug('*** open(%s, %d)', path, flags)

		if not self._exists( path ):
			return -errno.ENOENT

		with self.fileWrittenMutex:
			if not path in self.fileWritten:
				self.fileWritten[path] = False

		tempFile = self.getTempFile( path )
		if tempFile != None:
			if os.path.exists( tempFile ):
				return 0

		try:
			_logger.debug('file %s downloading...', path)
			_temp = self.tempFileName( path )
			if _temp != None:
				with self.getFsMutex( path ):
					self.fs.downloadFile( path, _temp )
			else:
				return -errno.ENOENT
			tempFile = self.getTempFile( path )
			if tempFile != None:
				_logger.debug('file %s downloaded', path)
			else:
				return -errno.ENOENT
		except:
			_logger.debug('file %s download: something failed', path)
			return -errno.ENOENT

		return 0

	# In this example this method is the same as open(). This method
	# is called by close() syscall, it's means that if open() maintain
	# an opened-file list, or lock files, or something... this method
	# must do reverse operation (refresh opened-file list, unlock files...
	def release ( self, path, flags ):
		_logger.debug('*** release(%s, %d)', path, flags)
		with self.fileWrittenMutex:
			isWritten = path in self.fileWritten
		if isWritten:
			_logger.debug('*** tempFile: %s', self.getTempFile( path ))
			if DELAYEDUPLOADS:
				if path in self.uploads:
					uploading = False
					with self.uploads[path].mutex:
						if self.uploads[path].status == "waiting":
							return 0
						if self.uploads[path].status == "uploading":
							uploading = True

					if uploading:
						self.uploads[path].join()
				uploadThread = Upload(path, self.getTempFile( path ), self )
				uploadThread.start()
				self.uploads.append( uploadThread )
			else:
				with self.getFsMutex( path ):
					self.fs.uploadFile( path, self.getTempFile( path ) )
				with self.fileWrittenMutex:
					self.fileWritten[path] = False
		with self.notUploadedMutex:
			if path in self.notUploaded:
				self.notUploaded.remove( path )
		_logger.debug('*** release finished(%s, %d)', path, flags)
		return 0

	def read ( self, path, length, offset ):
		_logger.debug('*** read(%s, %d, %d)', path, length, offset)

		fileName = self.getTempFile( path )

		if fileName == None:
			return -errno.EIO

		_file = open(fileName, "r")
		_logger.debug('*** read: opened temp file for read (%s, %d, %d)', path, length, offset)
		_file.seek(offset)
		_read = _file.read(length)
		_file.close()
		_logger.debug('*** read finished(%s, %d, %d)', path, length, offset)
		return _read

	def rmdir ( self, path ):
		_logger.debug('*** rmdir(%s)', path)
		with self.getFsMutex( path ):
			self.fs.remove( path )
		self.removeFromDirCache( path )
		self.dirCache.remove( path )
		self.statCache.remove( path )
		_logger.debug('*** rmdir(%s) finished', path)

	def truncate ( self, path, size ):
		_logger.debug('*** truncate(%s, %d)', path, size)
		fileName = self.getTempFile(path)

		if fileName == None:
			return -errno.EIO

		_file = open(fileName, "r+")
		_file.truncate(size)
		_file.close()
		with self.fileWrittenMutex:
			self.fileWritten[path] = True
		self.statCache.remove( path )
		_logger.debug('*** truncate finished(%s, %d)', path, size)
		return 0

	def unlink ( self, path ):
		_logger.debug('*** unlink(%s)', path)
		fileName = self.getTempFile( path )
		if fileName != None:
			os.remove( fileName )
		with self.notUploadedMutex:
			if path in self.notUploaded:
				self.notUploaded.remove(path)
		with self.getFsMutex( path ):
			self.fs.remove( path )
		with self.fileWrittenMutex:
			if path in self.fileWritten:
				del self.fileWritten[path]
		self.statCache.remove( path )
		self.removeFromDirCache( path )
		_logger.debug('*** unlink(%s) finished', path)
		return 0

	def write ( self, path, buf, offset ):
		_logger.debug('*** write(%s, %d)', path, offset)
		fileName = self.getTempFile(path)

		if fileName == None:
			return -errno.EIO

		_file = open(fileName, "r+")
		_file.seek(offset)
		_file.write(buf)
		_file.close()
		with self.fileWrittenMutex:
			self.fileWritten[path] = True
		self.statCache.remove( path )
		_logger.debug('*** write finished(%s, %d)', path, offset)
		return len(buf)

	def rename ( self, oldPath, newPath ):
		_logger.debug('*** rename(%s, %s)', oldPath, newPath)

		oldTempFile = self.getTempFile( oldPath )
		self.statCache.remove( newPath )
		self.statCache.remove( oldPath )
		self.removeFromDirCache( oldPath )
		self.addToDirCache( newPath )

		with self.inodesMutex:
			if oldPath in self.inodes:
				self.inodes[newPath] = self.inodes[oldPath]
				del self.inodes[oldPath]

		with self.notUploadedMutex:
			isNotUploaded = oldPath in self.notUploaded
		if isNotUploaded:
			if oldTempFile != None:
				os.rename( oldTempFile, self.hashFileName( newPath ) )
			else:
				_logger.debug('*** rename(%s, %s) failed: cant find old file temp file', oldPath, newPath)
				self.dirCache.remove( self.getParent(oldPath) )
				self.dirCache.remove( self.getParent(newPath) )
				return -errno.ENOENT
		else:
			with self.getFsMutex( oldPath ):
				ren = self.fs.rename( oldPath, newPath )
			if ren:
				if oldTempFile != None:
					newTempFile = self.tempFileName( newPath )
					if newTempFile != None:
						os.rename( oldTempFile, newTempFile )
					else:
						_logger.debug('*** rename(%s, %s) failed: cant find new file temp file', oldPath, newPath)
						self.dirCache.remove( self.getParent(oldPath) )
						self.dirCache.remove( self.getParent(newPath) )
						return -errno.EIO
				return 0
			else:
				_logger.debug('*** rename(%s, %s) failed: underlying fs call failed', oldPath, newPath)
				self.dirCache.remove( self.getParent(oldPath) )
				self.dirCache.remove( self.getParent(newPath) )
				return -errno.EIO

	def statfs ( self ):
		return self.fs.statFs()
