#include "lwserverthread.h"

/**
        @short LWServerThread constructor

	LWServerThread is the class that creates the thread that will
	allow multiple connections to the server (LWServer)
*/
LWServerThread::LWServerThread(QString name, QString clientAddress)
{
	this->clientAddress = clientAddress;
	this->_isAdministrator = false;

	QSettings settings(CONFIG_FILE, QSettings::IniFormat);

	QString host = settings.value( "Server/MySqlServerAddress", "localhost" ).toString();
	int port = settings.value( "Server/MySqlServerPort", 3306 ).toInt();
	QString username = settings.value( "Server/MySqlUsername", "root" ).toString();
	QString password = settings.value( "Server/MySqlPassword" ).toString();

	qDebug("Connecting to MySql database (server: %s:%d, user: %s, password: %s)...", host.toUtf8().data(), port, username.toUtf8().data(), password.toUtf8().data());

	mySqlInfo = new MySqlInfo( host, port );
	if (mySqlInfo->authenticate( username, password, name ) )
	{
		qDebug("MySql server authentication succesful");
		mySqlAuthenticated = true;
	} else {
		qDebug("MySql server authentication failed");
		mySqlAuthenticated = false;
	}
}

LWServerThread::~LWServerThread()
{
	mySqlInfo->close();
	delete mySqlInfo;
}

QByteArray LWServerThread::dispatchByteArrayMessage( QByteArray question )
{
	if ( ! username.isNull() )
	{
		LWServerCommand lwServerCommand( mySqlInfo, username, password, clientAddress, _isAdministrator );
		qDebug("Called command: %s", question.data());

		QByteArray response = lwServerCommand.doResponseByteArray(question);
		unsigned int responseSize = response.size();

		QByteArray resSize;
		resSize.append( (unsigned char) (responseSize >> 24) & 0xFF );
		resSize.append( (unsigned char) (responseSize >> 16) & 0xFF );
		resSize.append( (unsigned char) (responseSize >> 8) & 0xFF );
		resSize.append( (unsigned char) responseSize & 0xFF );

		qDebug("Dispatching binary array message of %d bytes", response.size());

		return resSize + response;
	} else {
		QByteArray response = "Not authenticated";
		return response;
	}
}

string LWServerThread::dispatchMessage( QByteArray question )
{
	qDebug("Ready to read %d bytes", question.size());

	qDebug("Question: %s", question.data());

	QRegExp sep("[?&]");
	QString command = QString(question).trimmed().section('?', 0, 0);
	QString parameter = QString(question).trimmed().section(sep, 1, 1);

	if ( command == "login" )
	{
        if ( qgetenv("DEBUG_NOAUTH") =="1" )
        {
            QString debugResponse;
            this->_isAdministrator = true;
            this->username = parameter;
            debugResponse = "Authentication ok.\n";
            return debugResponse.toStdString();
        }
        QString adminGroup = QString(question).trimmed().section('&', 1, 1);
        password = QString(question).trimmed().section('&', 2);

		if ( validate(parameter, password) )
		{
			this->username = parameter;
			QString response;
			qDebug("Admin group: %s", adminGroup.toUtf8().data());
			if ( adminGroup.isEmpty() || isAdministrator( this->username, adminGroup ) )
			{
				this->_isAdministrator = true;
				response = "Authentication ok.\n";
			} else {
				this->_isAdministrator = false;
				response = "Error: Not in admin group\n";
			}
            qDebug() << response;
			return response.toStdString();
		} else {
			QString response = "Error: Bad authentication\n";
			qDebug() << response;
			return response.toStdString();
		}
	}

	// set_server_information?<token>&<serverPort>&<serverType>&<status>
	// Example: set_server_information?7hf93hfh4h9f4hf34jdkhdfvnfs3432&storage_server&1
	if (command == "set_server_information")
	{
		QString token = QString(question).trimmed().section(sep, 1, 1);
		QString serverType = QString(question).trimmed().section(sep, 2, 2);
		QString status = QString(question).trimmed().section(sep, 3);
		QString ret = LWServerCommand::setServerInformation( mySqlInfo, token, serverType, status, clientAddress );
		ret = ret + "\n";
		qDebug("Response: %s", ret.toUtf8().data());
		return ret.toStdString();
	}

	if ( ! username.isNull() )
	{
		LWServerCommand lwServerCommand( mySqlInfo, username, password, clientAddress, _isAdministrator );
		qDebug("Called command: %s", question.data());

		QString response = lwServerCommand.doResponse(question) + "\n";
		qDebug("Response: %s", response.toUtf8().data());
		return response.toStdString();
	} else {
		QString response = "Not authenticated";
		return response.toStdString();
	}
}

bool LWServerThread::validate( QString username, QString password )
{
	if ( username.isEmpty() || password.isEmpty() ) // Don't allow empty user or password
	{
		return false;
	}

	if ( mySqlAuthenticated ) // Try mySql token authentication
	{
		qDebug("Trying MySql token authentication...");
		bool res = false;
		QStringList tokenList = mySqlInfo->getValues( "lw_user", "token", "username='" + username + "'" );
		qDebug("Checking token for user %s (%d values)", username.toUtf8().data(), tokenList.count());
		for ( int i = 0; i < tokenList.count(); i++ )
		{
			if ( tokenList[i] == password )
			{
				res = true;
			}
		}

		if ( res )
		{
			qDebug("MySql authentication succesful");
			return true;
		} else {
			qDebug("MySql authentication failed");
		}
	}

#ifdef Q_OS_LINUX
	qDebug("Trying PAM authentication...");
	return pamValidate( username.toUtf8().data(), password.toUtf8().data() );
#endif
#ifdef Q_OS_WIN
    qDebug("Trying Windows authentication...");
    return winValidate( username, password );
#endif
	return false;
}

#ifdef Q_OS_LINUX
struct pam_response * reply;

int functionConversation(int numMsg, const struct pam_message **msg, struct pam_response ** resp, void *appdata_ptr)  
{
    Q_UNUSED(appdata_ptr);
	for (int i = 0; i < numMsg; i++)
		qDebug("msg: %s", msg[i]->msg);
	*resp = reply;

	return PAM_SUCCESS;
}

bool LWServerThread::pamValidate( const char * username, const char * password )
{
	QString command = "/usr/bin/pamtester";
	QStringList arguments;
	arguments += "passwd";
	arguments += username;
	arguments += "authenticate";
	QProcess process;
	process.setReadChannel( QProcess::StandardError );
	qDebug("Launching command: %s %s", command.toUtf8().data(), arguments.join(" ").toUtf8().data());
	process.start( command, arguments );
	qDebug("Waiting for started");
	process.waitForStarted();
	qDebug("Wait for ready read");
        process.waitForReadyRead();
        QByteArray startText = process.readAllStandardError();

	qDebug(startText.data());

	if (startText.startsWith("Password:"))
	{
		qDebug("Detected password ask");
		process.write( password + QString("\n").toUtf8() );
		process.waitForBytesWritten();
	} else {
		qDebug("Weird");
		process.close();
		return false;
	}
	qDebug("Waiting for finished...");
	process.waitForFinished();
	qDebug("Finished.");
	return ( process.exitStatus() == QProcess::NormalExit && process.exitCode() == 0 );
/*
	bool res = true;

	reply = (struct pam_response *)malloc(sizeof(struct pam_response));

	qDebug("Username: %s, password: %s", username, password);
	//qDebug("Username: %s", username);

	const struct pam_conv local_conversation = { functionConversation, NULL };
	pam_handle_t *local_auth_handle = NULL; // this gets set by pam_start

	int retval;
	retval = pam_start("check_user", username, &local_conversation, &local_auth_handle);

	if (retval != PAM_SUCCESS)
	{
		return false;
	}

	reply[0].resp = strdup( password );
	reply[0].resp_retcode = 0;
	retval = pam_authenticate(local_auth_handle, 0);
	pam_set_item(local_auth_handle, PAM_RUSER, username);

	if (retval != PAM_SUCCESS)
	{
		return false;
	}

	retval = pam_end(local_auth_handle, retval);

	if (retval != PAM_SUCCESS)
	{
		res = false;
	}

	return res;
*/
}
#endif
#ifdef Q_OS_WIN
bool LWServerThread::winValidate( QString username, QString password)
{
    HANDLE token;
	return LogonUser((LPCWSTR)username.utf16(), NULL, (LPCWSTR)password.utf16(), LOGON32_LOGON_NETWORK, LOGON32_PROVIDER_DEFAULT, &token);
}
#endif

bool LWServerThread::isAdministrator( QString username, QString adminGroup )
{
    if ( adminGroup.isEmpty() )
    {
        return false;
    }

    return LWUserCommand::isUserInGroup( username, adminGroup );
}
