#include "lwsystemcommand.h"

QString LWSystemCommand::mountChroot(QString chrootHome, QString chrootMount, QString username, QString password, QString homePath, QString sambaServer)
{
	QString command = "/usr/sbin/mountchroot";
	QStringList arguments;
	arguments += chrootHome;
	arguments += chrootMount;
	arguments += username;
	arguments += homePath;
	arguments += sambaServer;

	QString credentialsFileName;

	QTemporaryFile credentials;
	credentials.setAutoRemove( true );
	credentials.setPermissions( QFile::ReadOwner | QFile::WriteOwner );
	if ( credentials.open() )
	{
		credentialsFileName = credentials.fileName();
		QTextStream out(&credentials);
		out << "username=" << username << "\n";
		out << "password=" << password << "\n";
		credentials.close();
	}

	arguments += credentialsFileName;

	qDebug("Running command: %s %s", command.toUtf8().data(), arguments.join(" ").toUtf8().data() );

	int ret = QProcess::execute( command, arguments );
	if ( ret == 0 )
		return QString("Ok");
	else
		return QString("Error %1").arg(ret);
}

QString LWSystemCommand::umountChroot(QString username)
{
	QString command = "/usr/sbin/umountchroot";
	QStringList arguments;
	arguments += username;

	bool ret = QProcess::startDetached( command, arguments );
	if ( ret )
		return QString("Ok");
	else
		return QString("Error %1").arg(ret);
}

QString LWSystemCommand::killSession(QString username)
{
#ifdef Q_OS_WIN
    PWTS_SESSION_INFO_1W buffer;
    DWORD pLevel = 1;
    DWORD pCount = 0;
    DWORD sessionId = -1;
    bool ret = WTSEnumerateSessionsEx(WTS_CURRENT_SERVER_HANDLE, &pLevel, 0, &buffer, &pCount);
    if (ret)
    {
        PWTS_SESSION_INFO_1W tmpBuffer = buffer;
		for (DWORD i = 0; i < pCount; i++)
        {
            QString user = QString::fromUtf16((const ushort *)tmpBuffer->pUserName);
            if (user.toLower() == username.toLower())
            {
                sessionId = tmpBuffer->SessionId;
                i = pCount;
            }
            tmpBuffer++;
        }
        if (sessionId != -1)
        {
            WTSLogoffSession(WTS_CURRENT_SERVER_HANDLE, sessionId, TRUE);
        }
    }
    WTSFreeMemoryExW( WTSTypeSessionInfoLevel1, buffer, pCount );
    buffer = NULL;
#endif
    return QString::null;
}

QString LWSystemCommand::getSystemMemory()
{
#ifdef Q_OS_LINUX
    meminfo();

    QString totalMemory = QString::number(kb_main_total);
    QString usedMemory = QString::number(kb_main_total - kb_main_free - kb_main_buffers - kb_main_cached);
#endif
#ifdef Q_OS_WIN
    MEMORYSTATUSEX statex;

    statex.dwLength = sizeof (statex);

    GlobalMemoryStatusEx (&statex);

    QString totalMemory = QString::number(statex.ullTotalPhys);
    QString usedMemory = QString::number(statex.ullTotalPhys - statex.ullAvailPhys);
#endif
    return usedMemory + "/" + totalMemory;
}

QString LWSystemCommand::getSystemDiskSpace()
{
    QString res;
#ifdef Q_OS_LINUX
	QProcess df;
	QString command = "df";
	QStringList arguments;
	arguments << "-B" << "1";
	arguments << "-t" << "ext2";
	arguments << "-t" << "ext3";
	arguments << "-t" << "ext4";
	arguments << "-t" << "btrfs";
	arguments << "-t" << "reiserfs";
	arguments << "-t" << "reiser4";
	arguments << "-t" << "f2fs";
	arguments << "-t" << "jfs";
	arguments << "-t" << "xfs";
	arguments << "-t" << "zfs";

	df.start( command, arguments );
	df.waitForStarted();
	df.waitForFinished();

	bool firstLine = true;
	bool titleLine = true;
	QByteArray line = df.readLine();
	while ( line.size() > 0 )
	{
		if ( ! titleLine )
		{
			if ( ! firstLine )
			{
				res += ",";
			} else {
				firstLine = false;
			}
			QString totalDisk = QString(line).simplified().section(" ", 1, 1);
			QString usedDisk = QString(line).simplified().section(" ", 2, 2);
			QString drive = QString(line).simplified().section(" ", 5, 100);
			res += "\"" + drive + "\"/" + usedDisk + "/" + totalDisk;
		} else {
			titleLine = false;
		}
		line = df.readLine();
	}
#endif
#ifdef Q_OS_WIN
    QStringList used;
    DWORD dwSize = MAX_PATH;
    LPWSTR szLogicalDrives = new WCHAR[MAX_PATH]; // = {0};
    DWORD dwResult = GetLogicalDriveStrings(dwSize, szLogicalDrives);

    if (dwResult > 0 && dwResult <= MAX_PATH)
    {
        LPWSTR szSingleDrive = szLogicalDrives;
        while(*szSingleDrive)
        {
            QString drive = QString::fromWCharArray( szSingleDrive );
            if ( GetDriveType( szSingleDrive ) == DRIVE_FIXED )
            {
                used += drive;
            }
            szSingleDrive += 4;
        }
    }
    delete szLogicalDrives;
    for (int i = 0; i < used.count(); i++)
    {
        DWORD dwSectPerClust;
        DWORD dwBytesPerSect;
        DWORD dwFreeClusters;
        DWORD dwTotalClusters;
        if (GetDiskFreeSpace((LPCWSTR)used[i].utf16(), &dwSectPerClust, &dwBytesPerSect, &dwFreeClusters, &dwTotalClusters) )
        {
            if (i != 0)
            {
                res += ",";
            }
            QString totalDisk = QString::number((qulonglong) dwTotalClusters * (qulonglong) dwSectPerClust * (qulonglong)dwBytesPerSect );
            QString usedDisk = QString::number((((qulonglong)dwTotalClusters - (qulonglong)dwFreeClusters) * (qulonglong)dwSectPerClust * (qulonglong)dwBytesPerSect) );
            res += "\"" + used[i] + "\"/" + usedDisk + "/" + totalDisk;
        }
    }
#endif
    return res;
}

QString LWSystemCommand::getSystemLoad()
{
    double load = 0.0;
#ifdef Q_OS_LINUX
    double loadAvg[1];

    int ok = getloadavg(loadAvg, 1);
    long numCpu = sysconf( _SC_NPROCESSORS_ONLN );
    load = 100 * loadAvg[0] / numCpu;


    if (numCpu <= 0 || ok == -1)
    {
        return "-1";
    }

    if (load <= 100)
    {
        return QString::number(load * 0.8, 'f', 2);
    } else {
        if (load <= 400)
        {
            return QString::number(80.0 + 20.0 * ( (load - 100) / 300), 'f', 2);
        } else {
            return QString("100.0");
        }
    }
#endif
#ifdef Q_OS_WIN
    FILETIME idleTime, kernelTime, userTime;
    FILETIME idleTime2, kernelTime2, userTime2;

    if ( GetSystemTimes(&idleTime, &kernelTime, &userTime) != 0 )
    {
        SleepEx( 250, FALSE );
        if ( GetSystemTimes(&idleTime2, &kernelTime2, &userTime2) != 0 )
        {
            ULONG _idle =  FileTimeToULong( idleTime2 ) - FileTimeToULong( idleTime );
            ULONG _kernel =  FileTimeToULong( kernelTime2 ) - FileTimeToULong( kernelTime );
            ULONG _user = FileTimeToULong( userTime2 ) - FileTimeToULong( userTime );
            double didle = (double) ( _idle );
            double dkernel = (double) ( _kernel );
            double duser = (double) ( _user );
            if ( dkernel + duser > 0.0 )
            {
                load = 100.0 * ( dkernel + duser - didle ) / ( dkernel + duser );
            } else {
                load = 0.0;
            }
        } else {
            qDebug("Something failed getting system times");
        }
    } else {
        qDebug("Something failed getting system times");
    }

    return QString::number( load, 'f', 2);
#endif
}

/* cpuWeight: double number, from 0.0 to 1.0. Higher values give more importance to cpu
   use when selecting preferred server
*/
QString LWSystemCommand::getPreferredServer( QList<double> cpuUsage, QList<double> freeMemory, QStringList serverId, double cpuWeight )
{
	int count = serverId.count();

	double bestResult = -1.0;
	QString servId;

	for (int i = 0; i < count; i++)
	{
		double cpuUse = cpuUsage[i];
		double freeMem = freeMemory[i];

		double result = (100.0 - cpuUse * cpuWeight) * freeMem;
		if ( result > bestResult )
		{
			servId = serverId[i];
			bestResult = result;
		}
	}

	return servId;
}

#ifdef Q_OS_WIN
ULONG LWSystemCommand::FileTimeToULong(const FILETIME & ft)
{
    ULARGE_INTEGER uli = { 0 };
    uli.LowPart = ft.dwLowDateTime;
    uli.HighPart = ft.dwHighDateTime;

    return uli.QuadPart;
}
#endif
