#include <stdlib.h>
#include <signal.h>

#include <QApplication>
#include <QCoreApplication>
#include <QObject>
#include <QString>
#include <QStringList>
#include <QHostAddress>

#include "lwserver.h"
#include "initdialog.h"
LWServer * server;

#include "constants.h"

QString error = QString::null;

void usage(char * command)
{
	printf("usage: %s <options>\n", command);
	printf("Options:\n");
	printf("  -a, --address ADDRESS        Serve on IP address ADDRESS\n");
	printf("  -p, --port PORT              Serve on port number PORT\n");
	printf("  -i, --init                   Initialize server (GUI dialog)\n");
	printf("  -u, --adminuser ADMINUSER    Initialize server with administrator ADMINUSER\n");
	printf("  -w, --password ADMINPASSWORD Initialize server with administrator password ADMINPASSWORD\n");
	printf("When initializing server you must specify ADMINUSER, PASSWORD and ADDRESS\n");
}

QString checkArguments(int argc, char ** argv, QString parameter1, QString parameter2, QString defaultValue = QString::null)
{
	QStringList validArguments;
	validArguments << "-a" << "--address" << "-p" << "--port" << "-u" << "--adminuser" << "-w" << "--password";

	QStringList noParameterArguments;
	noParameterArguments << "-i" << "--init";
	if (noParameterArguments.contains( parameter1 ) || noParameterArguments.contains( parameter2 ))
	{
		QString ret = "false";
		for (int i=1; i<argc; i++)
		{
			if (argv[i] == parameter1 || argv[i] == parameter2)
			{
				ret = "true";
			}
		}
		return ret;
	}

	for (int i=1; i<argc; i++)
	{
		if (validArguments.indexOf(argv[i]) == -1)
		{
			error = QObject::tr("Invalid option. Check parameters.");
		}

		if (argv[i] == parameter1 || argv[i] == parameter2)
		{
			if (i + 1 < argc)
			{
				return QString(argv[i+1]);
			} else {
				error = QObject::tr("The parameter needs a value.");
				return QString::null;
			}		
		} else {
			if (! noParameterArguments.contains( argv[i] ) )
			{
				i++;
			}
		}
	}
	return defaultValue;
}

void finish(int signal)
{
	if (signal == SIGINT) // CTRL-C pressed
	{
		printf("%s\n", QObject::tr("Stopping LibreWork server...").toUtf8().data());
		delete server;
		printf("%s\n", QObject::tr("done.").toUtf8().data());
		exit(0);
	}
}

int main(int argc, char ** argv)
{
	signal(SIGINT, finish);

	int port = DEFAULT_PORT;
	QHostAddress address = QHostAddress::Any;

	QString strport = checkArguments(argc, argv, "--port", "-p", QString::number(DEFAULT_PORT));
	QString straddress = checkArguments(argc, argv, "--address", "-a", DEFAULT_ADDRESS);
	QString adminuser = checkArguments(argc, argv, "--adminuser", "-u", QString::null);
	QString password = checkArguments(argc, argv, "--password", "-w", QString::null);
	QString init = checkArguments(argc, argv, "--init", "-i", "false");

	if (! adminuser.isEmpty() || ! password.isEmpty() || init == "true")
	{
		if ( init == "true" )
		{
			QApplication app(argc, argv);
			app.setOrganizationName("Tuxum Secure Systems");
			app.setApplicationName("Librework Server");
			InitDialog * initDialog = new InitDialog(adminuser, password);
			return ( initDialog->exec() == QDialog::Rejected );
		} else {
			if ( straddress == DEFAULT_ADDRESS )
			{
				error = "You must specify a local address when initializing with admin user and password";
			}
			if ( adminuser.isEmpty() || password.isEmpty() )
			{
				error = "You must specify both admin user and admin password when initializing server";
			}
		}
	}

	if (error != QString::null)
	{
		printf("%s\n", error.toUtf8().data());
		usage(argv[0]);
		exit(1);
	}

	if (!strport.isNull())
		port = strport.toInt();

	if (!straddress.isNull())
		address = QHostAddress(straddress);

	QCoreApplication app(argc, argv);
	app.setOrganizationName("Tuxum Secure Systems");
	app.setApplicationName("Librework Server");

	LWServer server(adminuser, password);

#ifdef Q_OS_LINUX
	/* Set LANGUAGE=en for pamtester executable */
	setenv("LANGUAGE", "EN", 1);
#endif

	int ret = server.listenConnections(address.toString().toUtf8().data(), port);
	if (  ret != 0 )
	{
		printf("%s. Error %d: %s\n", QObject::tr("Unable to start LibreWork server").toUtf8().data(), ret, server.lastError().toUtf8().data());
	}

	return ret;
}
